@@ -25,10 +25,14 @@
 
 (define latex-language "english")
 (define latex-style "generic")
-(define latex-style-hyp 'generic-style%)
 (define latex-packages '())
-(define latex-amsthm-hyp 'no-amsthm-package%)
+(define latex-virtual-packages '())
+(define latex-all-packages '())
+(define latex-texmacs-style "generic")
+(define latex-texmacs-packages '())
+(define latex-dependencies '("generic"))
 
+(define latex-packages-option (make-ahash-table))
 (define latex-uses-table (make-ahash-table))
 (define latex-catcode-table (make-ahash-table))
 (define latex-macro-table (make-ahash-table))
@@ -36,26 +40,58 @@
 (define latex-preamble-table (make-ahash-table))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Setting global parameters
+;; Setting and testing global parameters
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (tm-define (latex-set-language lan)
   (set! latex-language lan))
 
 (tm-define (latex-set-style sty)
-  (if (list? latex-style)
-    (set! latex-style (append (cDr latex-style) (list sty)))
-    (set! latex-style sty))
-  (set! latex-style-hyp (string->symbol (string-append sty "-style%"))))
+  (set! latex-style sty)
+  (latex-set-dependencies))
 
 (tm-define (latex-set-packages ps)
   (set! latex-packages ps)
-  (when (in? "amsthm" ps)
-    (set! latex-amsthm-hyp 'amsthm-package%)))
+  (latex-set-dependencies))
+
+(tm-define (latex-set-virtual-packages ps)
+  (set! latex-virtual-packages ps)
+  (latex-set-dependencies))
+
+(tm-define (latex-set-texmacs-style sty)
+  (set! latex-texmacs-style sty))
+
+(tm-define (latex-set-texmacs-packages l)
+  (set! latex-texmacs-packages l))
+
+(define (latex-set-dependencies)
+  (set! latex-all-packages
+        (list-remove-duplicates (append latex-packages latex-virtual-packages)))
+  (set! latex-dependencies
+        (latex-packages-dependencies (cons latex-style latex-all-packages))))
+
+(tm-define (latex-has-style? sty)
+  (== sty latex-style))
+
+(tm-define (latex-has-package? p)
+  (in? p latex-packages))
+
+(tm-define (latex-has-texmacs-style? sty)
+  (== sty latex-texmacs-style))
+
+(tm-define (latex-has-texmacs-package? p)
+  (in? p latex-texmacs-packages))
+
+(tm-define (latex-depends? p)
+  (in? p latex-dependencies))
 
 (tm-define (latex-book-style?)
   (in? latex-style '("book")))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Catcode generation
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (define (latex-catcode-defs-char c)
   (let* ((s (string-convert (list->string (list c)) "Cork" "UTF-8"))
          (r (string-convert s "UTF-8" "LaTeX")))
@@ -67,13 +103,47 @@
         ((list? doc) (for-each latex-catcode-defs-sub doc))))
 
 (define (latex-catcode-defs-char* c)
-  (if (in? c '(#\< #\> #\|))
+  (if (in? c '(#\< #\>))
     (ahash-set! latex-catcode-table (string c)
                 (number->string (char->integer c)))))
 
-(define (latex-catcode-defs-sub* doc)
-  (cond ((string? doc) (for-each latex-catcode-defs-char* (string->list doc)))
-        ((list? doc) (for-each latex-catcode-defs-sub* doc))))
+(define (env? t x)
+  (and (list>0? t) (func? (car t) '!begin) (list>1? (car t)) (== x (cadar t))))
+
+(define (latex-is-math? t)
+  (or (func? t '!math)
+      (func? t '!eqn)
+      (env? t "equation")
+      (env? t "gather")
+      (env? t "multline")
+      (env? t "split")
+      (env? t "equation*")
+      (env? t "gather*")
+      (env? t "multline*")
+      (env? t "align")
+      (env? t "flalign")
+      (env? t "alignat")
+      (env? t "align*")
+      (env? t "flalign*")
+      (env? t "alignat*")))
+
+(define (latex-is-text? t)
+  (func? t 'text))
+
+(define (latex-is-verb? t)
+  ;; TODO: consider also macros which expect verbatim args
+  (or (func? t '!verb)      (func? t '!verbatim)
+      (func? t '!verbatim*) (func? t 'tmverbatim)))
+
+(define (latex-catcode-defs-sub* doc text?)
+  (cond ((and text? (string? doc))
+         (for-each latex-catcode-defs-char* (string->list doc)))
+        ((and (list? doc) (latex-is-text? doc))
+         (for-each (cut latex-catcode-defs-sub* <> #t) doc))
+        ((and (list? doc) (or (latex-is-math? doc) (latex-is-verb? doc)))
+         (for-each (cut latex-catcode-defs-sub* <> #f) doc))
+        ((list? doc)
+         (for-each (cut latex-catcode-defs-sub* <> text?) doc))))
 
 (define (latex-catcode-def key im)
   (string-append "\\catcode`\\" key "=\\active \\def" key "{" im "}\n"))
@@ -91,7 +161,7 @@
           (apply string-append l3))) "")
     (begin
       (set! latex-catcode-table (make-ahash-table))
-      (latex-catcode-defs-sub* doc)
+      (latex-catcode-defs-sub* doc #t)
       (let* ((l1 (ahash-table->list latex-catcode-table))
              (l2 (list-sort l1 (lambda (x y) (string<=? (car x) (car y)))))
              (keys (map car l2))
@@ -105,12 +175,34 @@
         (apply string-append l3)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Macro and environment expansion
+;; Subroutines for reading the database
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define (env-begin? x)
   (or (func? x '!begin) (func? x '!begin*)))
 
+(define (latex-texmacs-arity x)
+  (if (env-begin? x)
+      (latex-texmacs-arity
+       (string->symbol (string-append "begin-" (cadr x))))
+      (logic-ref latex-texmacs-arity% x)))
+
+(define (latex-needs? x)
+  (if (env-begin? x)
+      (latex-needs?
+       (string->symbol (string-append "begin-" (cadr x))))
+      (logic-ref latex-needs% x)))
+
+(define (latex-texmacs-option? x)
+  (if (env-begin? x)
+      (latex-texmacs-option?
+       (string->symbol (string-append "begin-" (cadr x))))
+      (logic-ref latex-texmacs-option% x)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Macro and environment expansion
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (define (latex-substitute t args)
   (cond ((number? t) (list-ref args t))
 	((== t '---) (car args))
@@ -126,16 +218,11 @@
   (if (npair? t) t
       (let* ((head  (car t))
 	     (tail  (map latex-expand-macros (cdr t)))
-	     (body  (logic-ref latex-texmacs-macro% head
-			     latex-style-hyp latex-amsthm-hyp))
-	     (arity (logic-ref latex-texmacs-arity% head
-			     latex-style-hyp latex-amsthm-hyp))
+	     (body  (smart-ref latex-texmacs-macro head))
+	     (arity (and body (latex-texmacs-arity head)))
 	     (env   (and (env-begin? head)
-			 (logic-ref latex-texmacs-environment% (cadr head)
-				  latex-style-hyp latex-amsthm-hyp)))
-	     (envar (and (env-begin? head)
-			 (logic-ref latex-texmacs-env-arity% (cadr head)
-				  latex-style-hyp latex-amsthm-hyp))))
+			 (smart-ref latex-texmacs-environment (cadr head))))
+	     (envar (and env (latex-texmacs-arity head))))
 	(cond ((and body (== (length tail) arity))
 	       (latex-substitute body t))
 	      ((and env (== (length tail) 1) (== (length (cddr head)) envar))
@@ -146,26 +233,28 @@
 ;; Compute macro and environment definitions
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (latex-expand-def t)
+(define (latex-expand-def t protect?)
+  (cond ((and protect? (number? t)) (set! t `(!group ,t)) (set! protect? #f))
+        ((and (not protect?) (func? t '!option) (set! protect? #t))))
   (cond ((== t '---) "#-#-#")
 	((number? t) (string-append "#" (number->string t)))
-	((func? t '!recurse 1) (latex-expand-def (cadr t)))
+	((func? t '!recurse 1) (latex-expand-def (cadr t) protect?))
 	((func? t '!translate 1)
 	 (translate-from-to (cadr t) "english" latex-language))
-	((list? t) (map latex-expand-def t))
+	((list? t) (map (cut latex-expand-def <> protect?) t))
 	(else t)))
 
 ;; TODO: to be rewrited with better factorisation
 (define (latex-macro-defs-sub t)
   (when (pair? t)
-    (for-each latex-macro-defs-sub (cdr t))
-    (let* ((body  (and
-                    (not (logic-ref latex-needs% (car t)))
-                    (logic-ref latex-texmacs-macro% (car t)
-                               latex-style-hyp latex-amsthm-hyp)))
-	   (arity (logic-ref latex-texmacs-arity% (car t)
-			   latex-style-hyp latex-amsthm-hyp))
-           (option (logic-ref latex-texmacs-option% (car t)))
+    (if (and (or (func? t 'newcommand) (func? t 'renewcommand))
+             (> (length t) 2))
+      (for-each latex-macro-defs-sub (cddr t))
+      (for-each latex-macro-defs-sub (cdr t)))
+    (let* ((body   (and (not (latex-needs? (car t)))
+                        (smart-ref latex-texmacs-macro (car t))))
+	   (arity  (and body (latex-texmacs-arity (car t))))
+           (option (and body (latex-texmacs-option? (car t))))
            (args   (if option (filter (lambda (x)
                                         (not (and (list? x)
                                                   (== (car x) '!option))))
@@ -174,16 +263,14 @@
       (when (and body (== (length args) arity))
         (if option (set! arity (+ 1 arity)))
 	(ahash-set! latex-macro-table (car t)
-		    (list arity (latex-expand-def body)))
+		    (list arity (latex-expand-def body #f)))
 	(latex-macro-defs-sub body)))
     (let* ((body  (and (env-begin? (car t))
-                       (not (logic-ref latex-needs% (string->symbol (cadar t))))
-                       (logic-ref latex-texmacs-environment% (cadar t))))
-	   (arity (and (env-begin? (car t))
-		       (logic-ref latex-texmacs-env-arity% (cadar t))))
-           (option (and (env-begin? (car t))
-                        (logic-ref latex-texmacs-option% (cadar t))))
-           (args   (and (env-begin? (car t))
+                       (not (latex-needs? (car t)))
+                       (smart-ref latex-texmacs-environment (cadar t))))
+	   (arity (and body (latex-texmacs-arity (car t))))
+           (option (and body (latex-texmacs-option? (car t))))
+           (args   (and body
                         (if option (filter (lambda (x)
                                              (not (and (list? x)
                                                        (== (car x) '!option))))
@@ -192,18 +279,17 @@
       (when (and body (== (length args) (+ arity 2)))
         (if option (set! arity (+ 1 arity)))
 	(ahash-set! latex-env-table (cadar t)
-		    (list arity (latex-expand-def body)))
+		    (list arity (latex-expand-def body #f)))
 	(latex-macro-defs-sub body)))
-    (with body (or (and
-                     (not (logic-ref latex-needs% (car t)))
-                     (logic-ref latex-texmacs-preamble% (car t)
-                                latex-style-hyp latex-amsthm-hyp))
+    (with body (or (and (not (latex-needs? (car t)))
+                        (smart-ref latex-texmacs-preamble (car t)))
 		   (and (env-begin? (car t))
-                        (not (logic-ref latex-needs% (string->symbol (cadar t))))
-                        (logic-ref latex-texmacs-env-preamble% (cadar t)
-                                   latex-style-hyp latex-amsthm-hyp)))
+                        (not (latex-needs? (car t)))
+                        (smart-ref latex-texmacs-env-preamble (cadar t))))
       (when body
-	(ahash-set! latex-preamble-table (car t) body)))))
+        (ahash-set! latex-preamble-table
+                    (if (env-begin? (car t)) (cadar t) (car t)) body)
+        (latex-macro-defs-sub body)))))
 
 (define (latex<=? x y)
   (if (symbol? x) (set! x (symbol->string x)))
@@ -226,7 +312,7 @@
 	 (e3 (map (cut cons '!newenvironment <>) e2))
 	 (p1 (ahash-table->list latex-preamble-table))
 	 (p2 (list-sort p1 (lambda (x y) (latex<=? (car x) (car y)))))
-	 (p3 (map cdr (map latex-expand-def p2))))
+	 (p3 (map cdr (map (cut latex-expand-def <> #f) p2))))
     (cons '!append (append c3 e3 p3))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -237,10 +323,10 @@
   (with option ""
     (if (and (list>1? body) (list? (car body)) (== (caar body) '!option))
       (begin
-        (set! option (serialize-latex (latex-expand-def (cadar body))))
+        (set! option (serialize-latex (latex-expand-def (cadar body) #f)))
         (set! option (string-append "[" option "]"))
         (set! body (cadr body))))
-    (set! body (serialize-latex (latex-expand-def body)))
+    (set! body (serialize-latex (latex-expand-def body #f)))
     (set! body (string-replace body "\n\n" "*/!!/*"))
     (set! body (string-replace body "\n" " "))
     (set! body (string-replace body "*/!!/*" "\n\n"))
@@ -253,10 +339,10 @@
   (with option ""
     (if (and (list>1? body) (list? (car body)) (== (caar body) '!option))
       (begin
-        (set! option (serialize-latex (latex-expand-def (cadar body))))
+        (set! option (serialize-latex (latex-expand-def (cadar body) #f)))
         (set! option (string-append "[" option "]"))
         (set! body (cadr body))))
-    (set! body (serialize-latex (latex-expand-def body)))
+    (set! body (serialize-latex (latex-expand-def body #f)))
     (set! body (string-replace body "\n\n" "*/!!/*"))
     (set! body (string-replace body "\n  " " "))
     (set! body (string-replace body "\n" " "))
@@ -278,22 +364,48 @@
 	((func? t '!newenvironment 3) (apply latex-env-def (cdr t)))
 	(else (serialize-latex t))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Package dependencies management
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define (latex-package-direct-dependencies p)
+  (logic-ref-list latex-depends% p))
+
+(define (insert-dependencies l p)
+  (if (null? p) l
+      (if (in? (car p) l)
+          (insert-dependencies l (cdr p))
+          (with deps (latex-package-direct-dependencies (car p))
+            (insert-dependencies (append l (list (car p)))
+                                 (append deps (cdr p)))))))
+
+(tm-define (latex-packages-dependencies ps)
+  (:synopsis "Determine all dependencies of packages @ps")
+  (insert-dependencies (list) ps))
+
+(define (non-redundant-package? p among)
+  (with c (latex-packages-dependencies (list-difference among (list p)))
+    (not (in? p c))))
+
+(tm-define (latex-packages-simplify ps)
+  (:synopsis "Remove all implied packages in package list @ps")
+  (list-filter ps (lambda (p) (non-redundant-package? p ps))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Compute usepackage command for a document
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define (latex-command-uses s)
   (with packlist (logic-ref-list latex-needs% s)
-    (when packlist
-      (if (string? packlist) (set! packlist (list packlist)))
-      (for-each (cut ahash-set! latex-uses-table <> #t) packlist))))
+    (for-each (cut ahash-set! latex-uses-table <> #t) packlist)))
 
 (define (latex-use-which-package l)
   (when (and (list? l) (nnull? l))
     (let ((x (car l)))
       (if (symbol? x) (latex-command-uses x))
       (if (and (list? x) (>= (length l) 2) (== (car x) '!begin))
-	  (latex-command-uses (string->symbol (cadr x))))
+	  (latex-command-uses
+           (string->symbol (string-append "begin-" (cadr x)))))
       (if (match? x '(!begin "enumerate" (!option :%1)))
 	  (ahash-set! latex-uses-table "enumerate" #t))
       (for-each latex-use-which-package (cdr l)))))
@@ -308,19 +420,44 @@
 (define (filter-packages l)
   (filter (lambda (x) (nin? x tmtex-provided-packages)) l))
 
+(define (filter-packages* l)
+  (filter (lambda (x) (nin? (cAr x) tmtex-provided-packages)) l))
+
+(define (make-use-package l)
+  (with po (ahash-ref latex-packages-option (cAr l))
+    (let* ((optl (if (not po) (cDr l) (append (cDr l) po)))
+           (opt  (apply string-append (list-intersperse optl ",")))
+           (sty  (cAr l)))
+      (with opts (if (== opt "") "" (string-append "["  opt "]"))
+        (string-append "\\usepackage" opts "{" sty "}\n")))))
+
 (define (latex-as-use-package l1)
-  (let* ((l2 (sort l1 latex-use-package-compare))
-	 (l3 (map force-string l2))
-         (l4 (filter-packages l3))
-	 (l5 (list-intersperse l4 ","))
-	 (s  (apply string-append l5)))
-    (if (== s "") "" (string-append "\\usepackage{" s "}\n"))))
+  (let* ((l2  (sort l1 latex-use-package-compare))
+	 (l3  (filter
+                (lambda (x)
+                  (and (string? x)
+                       (not (ahash-ref latex-packages-option x))))
+                l2))
+         (l3* (map (lambda (x)
+                     (map force-string x))
+                   (filter
+                     list>0?
+                     (map
+                       (lambda (x)
+                         (if (ahash-ref latex-packages-option x) (list x) x))
+                       l2))))
+         (l4  (filter-packages  l3))
+         (l4* (filter-packages* l3*))
+	 (l5  (list-intersperse l4 ","))
+	 (s   (apply string-append l5))
+         (s*  (apply string-append (map make-use-package l4*))))
+    (if (== s "") s* (string-append "\\usepackage{" s "}\n" s*))))
 
 (tm-define (latex-use-package-command doc)
   (:synopsis "Return the usepackage command for @doc")
   (set! latex-uses-table (make-ahash-table))
   (latex-use-which-package doc)
-  (let* ((l1 latex-packages)
+  (let* ((l1 latex-all-packages)
 	 (s1 (latex-as-use-package (list-difference l1 '("amsthm"))))
 	 (l2 (map car (ahash-table->list latex-uses-table)))
 	 (s2 (latex-as-use-package (list-difference l2 l1))))
@@ -330,13 +467,48 @@
 ;; Page size settings
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (latex-preamble-language lan)
-      (string-append "\\usepackage[" lan "]{babel}\n"))
-
 (define (latex-preamble-page-type init)
-  (let* ((page-type (ahash-ref init "page-type"))
-	 (page-size (logic-ref latex-paper-type% page-type)))
-    (if page-size `(!append (geometry ,page-size) "\n") "")))
+  (let* ((l0 (ahash-table->list init))
+         (l1 (map car l0))
+         (l2 (map cdr l0))
+         (l3 (map (cut logic-ref latex-paper-opts% <>) l1))
+         (l4 (map (lambda (key val)
+                    (cond ((not val) '())
+                          ((== key "page-type")
+                           (or (logic-ref latex-paper-type% val) '()))
+                          ((== key "page-orientation") val)
+                          ((string? key)
+                           (string-append key "= " (tmtex-decode-length val)))
+                          (else '()))) l3 l2))
+         (l5 (filter nnull? l4))
+         (page-opts (list-intersperse l5 ",")))
+    (if (nnull? page-opts)
+      `(!append (geometry (!concat ,@page-opts)) "\n") "")))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Color definitions
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (html-color->latex-xcolor s)
+  "Take an hexa html color string and return an hex triplet string"
+  (cond ((string-starts? s "#") (html-color->latex-xcolor (string-tail s 1)))
+        ((== 3 (string-length s))
+         (let ((r (substring s 0 1))
+               (g (substring s 1 2))
+               (b (substring s 2 3)))
+           (string-append r r g g b b)))
+        ((== 4 (string-length s)) (html-color->latex-xcolor (string-take s 3)))
+        ((== 6 (string-length s)) s)
+        ((== 8 (string-length s)) (string-take s 6))
+        (else s)))
+
+(define (latex-colors-defs colors)
+  (apply string-append
+         (map (lambda (x)
+                (string-append
+                  "\\definecolor{" x "}{HTML}{"
+                  (html-color->latex-xcolor (get-hex-color x)) "}\n"))
+              colors)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Building the preamble
@@ -345,19 +517,28 @@
 (define (latex-make-option l)
   (string-append "[" (apply string-append (list-intersperse l ",")) "]"))
 
-(tm-define (latex-preamble text style lan init)
+(define (set-packages-option pack opts)
+  (if (nnull? opts)
+    (ahash-set! latex-packages-option pack opts)))
+
+(tm-define (latex-preamble text style lan init colors colormaps)
   (:synopsis "Compute preamble for @text")
+  (set! latex-packages-option (make-ahash-table))
+  (set-packages-option "xcolor" colormaps)
   (let* ((Page         (latex-preamble-page-type init))
 	 (Macro        (latex-macro-defs text))
-	 (Text         (list '!tuple Page Macro text))
+	 (Colors       (latex-colors-defs colors))
+	 (Text         (list '!tuple Page Macro Colors text))
 	 (pre-page     (latex-serialize-preamble Page))
 	 (pre-macro    (latex-serialize-preamble Macro))
+	 (pre-colors   (latex-serialize-preamble Colors))
 	 (pre-catcode  (latex-catcode-defs Text))
 	 (pre-uses     (latex-use-package-command Text)))
     (values
-      (cond ((and (in? "amsthm" latex-packages)(== style "amsart")) "[amsthm]")
+      (cond ((and (in? "amsthm" latex-all-packages)
+                  (== style "amsart")) "[amsthm]")
             ((list? style) (latex-make-option (cDr style)))
             (else ""))
       (string-append pre-uses)
       (string-append pre-page)
-      (string-append pre-catcode pre-macro))))
+      (string-append pre-catcode pre-macro pre-colors))))
