@@ -25,8 +25,8 @@ string verbatim_escape (string s);
 * During the parsing, the following global variables are used:
 *
 *     command_type   Contains the types of all currently defined tex commands.
-*                    This is either 'command' 'modifier' 'operator'
-*                    'environment' 'list' 'symbol' 'big-symbol' or 'user'.
+*                    See latex-type in latex-drd.scm for the list of possible
+*                    types.
 *     command_arity  Contains the corresponding arity.
 *     command_def    Contains the definitions of user commands.
 *
@@ -49,7 +49,6 @@ struct latex_parser {
   bool unicode;
   char lf;
   bool pic;
-  bool keep_src;
   hashmap<string,bool> loaded_package;
   latex_parser (bool unicode2): level (0), unicode (unicode2) {}
   void latex_error (string s, int i, string message);
@@ -83,10 +82,10 @@ struct latex_parser {
 void
 latex_parser::latex_error (string s, int i, string message) {
   if (!textm_class_flag) {
-    cerr << "Latex error] " << message << "\n";
+    convert_error << "latex error, " << message << "\n";
     if (i>30) s= "..." * s (i-27, N(s));
     if (N(s)>60) s= s (0, 57) * "...";
-    cerr << "Latex error] in " << s << "\n";
+    convert_error << "latex error in " << s << "\n";
   }
 }
 
@@ -134,15 +133,15 @@ is_tex_alpha (string s) {
   return true;
 }
 
-static bool
+bool
 test_macro (string s, int i, string name) {
   while (i < N(s) && s[i] == ' ') i++;
   return test (s, i, name) &&
     (N(s) == i + N(name) || !is_tex_alpha (s[i+N(name)]));
 }
 
-static bool
-test_env (string s, int i, string name, bool end=false) {
+bool
+test_env (string s, int i, string name, bool end= false) {
   string tok= end? "\\end":"\\begin";
   int step= end? 4:6;
   if (!test_macro (s, i, tok)) return false;
@@ -295,6 +294,7 @@ latex_parser::parse (string s, int& i, string stop, int change) {
       else {
 	tree u= parse_backslash (s, i, change);
 	if (u != "") t << u;
+        if (u == tuple ("\\\n")) t << "\n";
       }
       break;
     case '\'':
@@ -345,7 +345,9 @@ latex_parser::parse (string s, int& i, string stop, int change) {
       break;
     case '{': {
       i++;
+      t << tree (TUPLE, "\\begingroup");
       t << parse (s, i, "}", change);
+      t << tree (TUPLE, "\\endgroup");
       if ((i<n) && (s[i]=='}')) i++;
 
       int ln=0;
@@ -502,14 +504,21 @@ latex_parser::parse_backslash (string s, int& i, int change) {
     if (i<n && s[i] == '{') { i++; u= parse (s, i, "}"); i++; }
     return tree (TUPLE, "\\href", ss, u);
   }
-  if (((i+3)<n) && (s(i,i+2)=="\\\n")) {i++; return "";}
 
   /************************ special commands *********************************/
   i++;
   if (i==n) return "";
   if (s[i]==' ') {
-    i++;
-    return tree (TUPLE, "\\ ");
+    while (i<n && (s[i] == ' ' || s[i] == '\t')) i++;
+    if (i<n && s[i] != '\n')
+      return tree (TUPLE, "\\ ");
+    else
+      return tree (TUPLE, "\\\n");
+  }
+  if (s[i]=='\n') {
+    while (i<n && s[i]=='\n') i++;
+    lf= 'N';
+    return tree (TUPLE, "\\\n");
   }
   if (!is_tex_alpha(s[i])) {
     i++;
@@ -588,7 +597,7 @@ latex_parser::parse_symbol (string s, int& i) {
   }
   while ((i<N(s)) && s[i] == ' ') i++;
   if (s[i] == '\n') {lf= 'N'; i++;}
-  return s(start, end);
+  return tree (TUPLE, s(start, end));
 }
 
 static bool
@@ -970,6 +979,8 @@ latex_parser::parse_command (string s, int& i, string cmd, int change) {
         if (arity>0) {
           i=j;
           tree st= parse_symbol (s, i);
+          if (cmd == "\\def" && arity == 2 && is_tuple (st) && N(st) == 1)
+            st= st[0];
           t << st;
           u << st;
           arity--;
@@ -1005,12 +1016,12 @@ latex_parser::parse_command (string s, int& i, string cmd, int change) {
       command_type  (var)= "user";
       command_arity (var)= 0;
       command_def   (var)= tree_to_str_array (u[2]);
-      // hack to retrive arity of shortcuts
+      // hack to retrieve arity of shortcuts
       if (is_func (t[2], TUPLE, 1) &&
           latex_type (as_string (t[2][0])) != "undefined") {
         command_arity (var)= latex_arity (as_string (t[2][0]));
       }
-      // hack to retrive shortcuts to environments
+      // hack to retrieve shortcuts to environments
       if (as_string (t[2]) == "\\begin" || as_string (t[2]) == "\\end") {
         command_arity (var)= 1;
         command_type  (var)= "begin-end!";
@@ -1040,18 +1051,18 @@ latex_parser::parse_command (string s, int& i, string cmd, int change) {
     if (is_tuple (t, "\\declaretheorem*", 2) ||
         is_tuple (t, "\\declaretheorem",  1)) {
       string var= "\\begin-" * string_arg (t[N(t)-1]);
-      command_type  (var)= "environment";
+      command_type  (var)= "enunciation";
       command_arity (var)= 0;
       var= "\\end-" * string_arg (t[N(t)-1]);
-      command_type  (var)= "environment";
+      command_type  (var)= "enunciation";
       command_arity (var)= 0;
     }
     if (is_tuple (t, "\\newtheorem", 2) || is_tuple (t, "\\newtheorem*", 2)) {
       string var= "\\begin-" * string_arg (t[1]);
-      command_type  (var)= "environment";
+      command_type  (var)= "enunciation";
       command_arity (var)= 0;
       var= "\\end-" * string_arg (t[1]);
-      command_type  (var)= "environment";
+      command_type  (var)= "enunciation";
       command_arity (var)= 0;
     }
     if (is_tuple (t, "\\SetKwData", 2) || is_tuple (t, "\\SetKwFunction", 2)) {
@@ -1196,6 +1207,7 @@ latex_parser::parse_command (string s, int& i, string cmd, int change) {
     else if (cmd(0, 5) == "\\end-")
       orig_cmd= "\\end{" * cmd(5, N(cmd)) * "}";
     string code= orig_cmd * s(begin_parse, i);
+    code= verbatim_escape (code);
     if (command_type ("!mode") == "math")
       t= tuple ("\\latex_preview", cmd(1, N(cmd)), compound ("text", code));
     else
@@ -1503,15 +1515,17 @@ accented_to_Cork (tree t) {
 	return ret_s;
       }
       if (s[1] == '\'') return "\001";
+      if (s[1] == '\"') return "\004";
+      if (s[1] == 'H' ) return "\005";
+      if (s[1] == 'r' ) return "\006";
+      if (s[1] == 'v' ) return "\007";
+      if (s[1] == 'u' ) return "\010";
+      if (s[1] == '=' ) return "\011";
+      if (s[1] == '.' ) return tuple ("\\math", tuple ("\\wide", "", "<dot>"));
+      if (s[1] == 'c' ) return "\013";
+      if (s[1] == 'k' ) return "\014";
       if (s[1] == '^' ) return "\136";
-      if (s[1] == '\"') return "\004"; // "
       if (s[1] == '~' ) return "\176";
-      if (s[1] == '=' ) return "\026";
-      if (s[1] == '.' ) return "\137";
-      if (s[1] == 'u' ) return "\025";
-      if (s[1] == 'v' ) return "\024";
-      if (s[1] == 'H' ) return "\175";
-      if (s[1] == 'c' ) return "\030";
     }
     else {
       char c1= v[0], c2= s[1];
@@ -1538,7 +1552,7 @@ latex_parser::parse (string s, int change) {
   // We first cut the string into pieces at strategic places
   // This reduces the risk that the parser gets confused
   array<string> a;
-  int i, start=0, cut=0, n= N(s);
+  int i, start=0, cut=0, n= N(s), count= 0;
   for (i=0; i<n; i++)
     if (s[i]=='\n' || (s[i] == '\\' && test (s, i, "\\nextbib"))) {
       while ((i<n) && is_space (s[i])) i++;
@@ -1549,18 +1563,19 @@ latex_parser::parse (string s, int change) {
 	i += 30;
 	start= i;
       }
-      else if (test_env   (s, i, "document")        ||
-               test_env   (s, i, "abstract")        ||
-               test_macro (s, i, "\\part")          ||
-               test_macro (s, i, "\\chapter")       ||
-               test_macro (s, i, "\\section")       ||
-               test_macro (s, i, "\\subsection")    ||
-               test_macro (s, i, "\\subsubsection") ||
-               test_macro (s, i, "\\paragraph")     ||
-               test_macro (s, i, "\\subparagraph")  ||
-               test_macro (s, i, "\\nextbib")       ||
-               test_macro (s, i, "\\newcommand")    ||
-               test_macro (s, i, "\\def")) {
+      else if (test_macro (s, i, "\\nextbib") || (count == 0 &&
+                (test_env   (s, i, "document")        ||
+                 test_env   (s, i, "abstract")        ||
+                 test_macro (s, i, "\\part")          ||
+                 test_macro (s, i, "\\chapter")       ||
+                 test_macro (s, i, "\\section")       ||
+                 test_macro (s, i, "\\subsection")    ||
+                 test_macro (s, i, "\\subsubsection") ||
+                 test_macro (s, i, "\\paragraph")     ||
+                 test_macro (s, i, "\\subparagraph")  ||
+                 test_macro (s, i, "\\nextbib")       ||
+                 test_macro (s, i, "\\newcommand")    ||
+                 test_macro (s, i, "\\def")))) {
         a << s (start, i);
         start= i;
         while (i < n && test_macro (s, i, "\\nextbib")) {
@@ -1595,7 +1610,13 @@ latex_parser::parse (string s, int change) {
           i= cut + 1;
         }
       }
+      else if (s[i] != '\n' && !(s[i] == '\\' && test (s, i, "\\nextbib")))
+        i--;
     }
+    else if ((i == 0 || s[i-1] != '\\') && s[i] == '{')
+      count++;
+    else if ((i == 0 || s[i-1] != '\\') && s[i] == '}')
+      count--;
   a << s (start, i);
 
   // We now parse each of the pieces
@@ -1637,16 +1658,16 @@ clean_latex_comments (string s) {
   string r = "";
   int start = 0, stop = 0;
   while (stop < N(s)) {
-    //cout << start << " : " << stop << "\n";
     stop = search_forwards ("%", stop, s);
-    //cout << s[stop -1] << s[stop ] << s[stop +1];
     if (stop == -1) {
       r << s (start, N(s));
       return r;
     }
     else if (stop == 0 || s[stop -1] != '\\') {
       r << s (start, stop) << "\n";
-      stop = search_forwards ("\n", stop, s) + 1;
+      stop = search_forwards ("\n", stop, s);
+      if (stop == -1) break;
+      stop++;
       start = stop;
       continue;
     }
@@ -1688,12 +1709,12 @@ get_latex_language (string s) {
       break;
   string r = s(start+1, stop+1);
 
-  tree langs = concat();
-  langs << "british" << "bulgarian" << "chinese" << "czech" << "danish"
-    << "dutch" << "finnish" << "french" << "german" << "hungarian" << "italian"
-    << "japanese" << "korean" << "polish" << "portuguese" << "romanian"
-    << "russian" << "slovene" << "spanish" << "swedish" << "taiwanese"
-    << "ukrainian";
+  tree langs = concat(); 
+  langs << "british" << "bulgarian" << "chinese" << "croatian" << "czech"
+        << "danish" << "dutch" << "finnish" << "french" << "german"
+        << "hungarian" << "italian" << "japanese" << "korean" << "polish"
+        << "portuguese" << "romanian" << "russian" << "slovene" << "spanish"
+        << "swedish" << "taiwanese" << "ukrainian";
 
   for (int i = 0 ; i < N(langs) ; i++)
     if (test(r, 0 , as_string(langs[i])))
@@ -1822,149 +1843,33 @@ latex_encoding_to_iconv (string s) {
   return "";
 }
 
-static bool
-paragraph_break_here (string s, int i) {
-  int n= N(s);
-  if (i >= n || s[i] != '\n') return false;
-  i++;
-  while (i < n && s[i] == ' ') i++;
-  if (i >= n || s[i] != '\n') return false;
-  return true;
-}
-
-static string
-add_paragraph_markup (string s) {
-  string r;
-  int i= 0, start= 0, n= N(s);
-  while (i < n) {
-    int j= i+1;
-    if ((s[i] == '\n' && paragraph_break_here (s, i)) ||
-        (j < n && s[j] == '\\' &&
-        // strategic places to cut
-        (test_env   (s, j, "abstract")          ||
-         test_env   (s, j, "abstract", false)   ||
-         test_env   (s, j, "document")          ||
-         test_env   (s, j, "document", false)   ||
-         test_env   (s, j, "titlepage")         ||
-         test_env   (s, j, "titlepage", false)  ||
-         test_macro (s, j, "\\chapter")         ||
-         test_macro (s, j, "\\date")            ||
-         test_macro (s, j, "\\maketitle")       ||
-         test_macro (s, j, "\\part")            ||
-         test_macro (s, j, "\\section")         ||
-         test_macro (s, j, "\\subsection")      ||
-         test_macro (s, j, "\\subsubsection")   ||
-         test_macro (s, j, "\\tableofcontents") ||
-         test_macro (s, j, "\\title")))) {
-      r << s(start, i+1)
-        << "\n\n\\textm@break{" << as_string (i) << "}\n";
-      start= i+1;
-      if (!(s[i] == '\n' && paragraph_break_here (s, i))) {
-        i+= 4;
-      }
-      else {
-        while (i < n) {
-          if (s[i] == ' ' || s[i] == '\n') i++;
-          else if (s[i] == '%' && (i == 0 || s[i-1] != '\\'))
-            while (i < n && s[i] != '\n') i++;
-          else break;
-        }
-      }
-    }
-    else
-      i++;
-  }
-  if (i!= start) r << s(start, n);
-  return r;
-}
-
-static tree
-clean_paragraph_markup (tree t, int d) {
-  if (is_atomic (t)) return t;
-  tree r(L(t));
-  bool merge= false;
-  for (int i=0; i<N(t); i++) {
-    if (is_tuple (t[i], "\\textm@break", 1)) {
-      if (d == 0) {
-        if (merge)
-          r[N(r)-1]= t[i];
-        else
-          r << t[i];
-        merge= true;
-      }
-    }
-    else {
-      r << clean_paragraph_markup (t[i], d + 1);
-      merge= false;
-    }
-
-    if (is_tuple (t[i]) && as_string (t[i][0])(0, 7) == "\\begin-"
-                        && as_string (t[i][0]) != "\\begin-document")
-      d++;
-    else if (is_tuple (t[i]) && as_string (t[i][0])(0, 5) == "\\end-"
-                             && as_string (t[i][0]) != "\\end-document")
-      d--;
-  }
-  return r;
-}
-
-static tree
-clean_paragraph_markup (tree t) {
-  return clean_paragraph_markup (t, 0);
-}
-
-static tree
-fill_paragraph_markup (tree t, string s) {
-  if (is_atomic (t)) return t;
-  tree r(L(t));
-  int start= 0, stop= 0;
-  for (int i=0; i<N(t); i++) {
-    if (is_tuple (t[i], "\\textm@break", 1)) {
-      stop= as_int (simplify_concat (t[i][1])) + 1;
-      r << tuple ("\\textm@break", verbatim_escape (s(start, stop)));
-      start= stop;
-    }
-    else
-      r << t[i];
-  }
-  if (!is_tuple (t[N(t)-1], "\\textm@break", 1))
-    r << tuple ("\\textm@break", verbatim_escape (s(start, N(s))));
-  return r;
-}
-
 tree
-parse_latex (string s, bool change, bool using_cork, bool as_pic, bool keep_src) {
+parse_latex (string s, bool change, bool as_pic) {
   tree r;
   s= dos_to_better (s);
-  string encoding= "Cork";
   string lan= get_latex_language (s);
-  if (!using_cork) {
-    encoding= latex_encoding_to_iconv (get_latex_encoding (s));
-    if (encoding != "UTF-8" && encoding != "Cork" && encoding != "")
-      s= convert (s, encoding, "UTF-8");
-    else if (encoding == "")
-      // migth be pure ascii TeX or TeX snippet:
-      // we then make heuristic charset detection
-      s= western_to_utf8 (s);
-  }
+  string encoding= latex_encoding_to_iconv (get_latex_encoding (s));
+  if (encoding != "UTF-8" && encoding != "Cork" && encoding != "")
+    s= convert (s, encoding, "UTF-8");
+  else if (encoding == "")
+    // migth be pure ascii TeX or TeX snippet:
+    // we then make heuristic charset detection
+    s= western_to_utf8 (s);
+
+  if (encoding != "Cork")
+    s= convert (s, "LaTeX", "UTF-8");
 
   latex_parser ltx (encoding != "Cork");
   ltx.lf= 'M';
   ltx.pic= as_pic;
-  ltx.keep_src= keep_src;
   string s1= s;
-  if (keep_src) s1= add_paragraph_markup (s);
   r= ltx.parse (s1, change?2:0);
-  if (keep_src) {
-    r= clean_paragraph_markup (r);
-    r= fill_paragraph_markup (r, s);
-  }
   r= accented_to_Cork (r);
   if (lan == "") return r;
   return compound ("!language", r, lan);
 }
 
 tree
-parse_latex_document (string s, bool change, bool as_pic, bool keep_src) {
-  return compound ("!file", parse_latex (s, change, false, as_pic, keep_src));
+parse_latex_document (string s, bool change, bool as_pic) {
+  return compound ("!file", parse_latex (s, change, as_pic));
 }
