@@ -481,19 +481,21 @@ init_std_drd () {
 	accessible (0) -> url_type (0) ->
 	accessible (1) -> regular (1));       // FIXME: location?
   init (SCRIPT, "script",
-	options (1, 1, BIFORM) -> returns_adhoc () ->
-	accessible (0) -> code (0) -> name (0, "script") ->
-	accessible (1) -> regular (1) -> name (0, "where"));
+	repeat (1, 1, BIFORM) -> returns_adhoc () ->
+	accessible (0) -> code (0) -> name (0, "function") ->
+	accessible (1) -> regular (1) -> name (0, "arguments"));
+  init (FIND_ACCESSIBLE, "find-accessible",
+	fixed (1) -> regular (0));
   init (HLINK, "hlink",
 	fixed (1, 1, BIFORM) ->
 	accessible (0) -> name (0, "text") ->
 	url_type (1) -> name (1, "destination") ->
 	name ("hyperlink"));
   init (ACTION, "action",
-	options (2, 1, DETAILED) ->
+	repeat (2, 1, DETAILED) ->
 	accessible (0) -> name (0, "text") ->
-	code (1) -> name (1, "script") ->
-	regular (2) -> name (2, "where"));
+	code (1) -> name (1, "function") ->
+	regular (2) -> name (2, "arguments"));
   init (SET_BINDING, "set-binding",
 	options (1, 2));                      // see env_exec.cpp
   init (GET_BINDING, "get-binding",
@@ -822,6 +824,9 @@ init_std_drd () {
   init_var (THE_LABEL, TYPE_REGULAR);
   init_var (THE_TAGS, TYPE_ADHOC);
   init_var (THE_MODULES, TYPE_ADHOC);
+  init_var (GLOBAL_TITLE, TYPE_STRING);
+  init_var (GLOBAL_AUTHOR, TYPE_STRING);
+  init_var (GLOBAL_SUBJECT, TYPE_STRING);
 
   init_var (FONT, TYPE_STRING);
   init_var (FONT_FAMILY, TYPE_STRING);
@@ -1042,6 +1047,7 @@ init_std_drd () {
   init_var (ORNAMENT_SHAPE, TYPE_STRING);
   init_var (ORNAMENT_TITLE_STYLE, TYPE_STRING);
   init_var (ORNAMENT_BORDER, TYPE_LENGTH);
+  init_var (ORNAMENT_SWELL, TYPE_NUMERIC);
   init_var (ORNAMENT_HPADDING, TYPE_LENGTH);
   init_var (ORNAMENT_VPADDING, TYPE_LENGTH);
   init_var (ORNAMENT_COLOR, TYPE_COLOR);
