@@ -401,6 +401,45 @@ iso_to_koi8uk (string s) {
   return r;
 }
 
+/******************************************************************************
+* Convert TS1 defined symbols to universal encoding
+******************************************************************************/
+
+tree
+convert_OTS1_symbols_to_universal_encoding (tree t) {
+  if (is_atomic (t)) return t;
+  if (N(t) == 0) {
+    static tree symbols (CONCAT);
+    if (N(symbols) == 0)
+      symbols << "cent" << "copyright" << "currency" << "yen" << "twosuperior"
+        << "threesuperior" << "onesuperior" << "mu" << "onequarter"
+        << "onehalf" << "threequarters" << "trademark";
+    tree l= tree (as_string (L(t)));
+    if (contains (l, A(symbols)))
+      return "<" * as_string (L(t)) * ">";
+    else if (l == "degreesign")
+      return "<degree>";
+    else if (l == "copyleft")
+      return "<copyright>"; // Copyleft is nor defined in TeXmacs universal
+                            // encoding, neither in utf8, neither buildable
+                            // with TeXmacs primitive construction.
+    else if (l == "registered")
+      return "<circledR>";
+    else if (l == "paragraphsign")
+      return "<paragraph>";
+    else if (l == "euro")
+      return "<#20AC>";
+    else
+      return t;
+
+  }
+  int i, n= N(t);
+  tree r (t, n);
+  for (i=0; i<n; i++)
+    r[i]= convert_OTS1_symbols_to_universal_encoding (t[i]);
+  return r;
+}
+
 /******************************************************************************
 * Convert between TeXmacs and XML strings
 ******************************************************************************/
@@ -539,7 +578,7 @@ static string hundreds[10]= {
 
 string
 roman_nr (int nr) {
-  if (nr<0) return "-" * roman_nr (nr);
+  if (nr<0) return "-" * roman_nr (-nr);
   if (nr==0) return "o";
   if (nr>1000) return "m" * roman_nr (nr-1000);
   if (nr==1000) return "m";
@@ -557,7 +596,7 @@ Roman_nr (int nr) {
 
 string
 alpha_nr (int nr) {
-  if (nr<0) return "-" * alpha_nr (nr);
+  if (nr<0) return "-" * alpha_nr (-nr);
   if (nr==0) return "0";
   if (nr<=26) return string ((char) (((int) 'a')+ nr-1));
   return alpha_nr ((nr-1)/26) * alpha_nr (((nr-1)%26)+1);
@@ -570,10 +609,11 @@ Alpha_nr (int nr) {
 
 string
 fnsymbol_nr (int nr) {
+  if (nr<0) nr= -nr;
   string sym, r;
   int i, m= (nr-1)%6, n= ((nr-1)/6)+1;
   switch (m) {
-    case 0: sym= "<ast>";        break;
+    case 0: sym= "<asterisk>";   break;
     case 1: sym= "<dag>";        break;
     case 2: sym= "<ddag>";       break;
     case 3: sym= "<paragraph>";  break;
@@ -787,6 +827,83 @@ tm_recompose (array<string> a) {
   return r;
 }
 
+int
+tm_search_forwards (string s, int pos, string in) {
+  int k= N(s), n= N(in);
+  if (k == 0) return pos;
+  char c= s[0];
+  while (pos+k <= n) {
+    if (in[pos] == c && test (in, pos, s)) return pos;
+    tm_char_forwards (in, pos);
+  }
+  return -1;
+}
+
+int
+tm_search_backwards (string s, int pos, string in) {
+  while (pos >= 0) {
+    if (test (in, pos, s)) return pos;
+    tm_char_backwards (in, pos);
+  }
+  return -1;
+}
+
+static array<string>
+tm_string_split_between_words (string s) {
+  int i= 0, j= -1, n= N(s);
+  char status= 'o';
+  array<string> r;
+  while (i < n && j < n/2) {
+    char c= s[i];
+    if      (is_numeric (c)   && status == 'c');
+    else if (is_iso_alpha (c) && status == 'a');
+    else {
+      if      (is_numeric   (c)) status= 'c';
+      else if (is_iso_alpha (c)) status= 'a';
+      else status= 'x';
+      j= i;
+    }
+    tm_char_forwards (s, i);
+  }
+  if (j > 0 && j < n)
+    r << s(0, j) << s(j, n);
+  else
+    r << s;
+  return r;
+}
+
+static array<string>
+tm_string_split_at_spaces (string s) {
+  int i= 0, j= 0, n= N(s);
+  array<string> r;
+  while (i>=0 && j < n/2) {
+    i= tm_search_forwards (" ", i, s);
+    if (i == -1) break;
+    j= i++;
+  }
+  if (j < 1 || j >= n)
+    r << s;
+  else if (j == n-1)
+    r << s(0, j) << s(j, n);
+  else
+    r << s(0, j) << s(j, j+1) << s(j+1, n);
+  return r;
+}
+
+array<string>
+tm_string_split (string s) {
+  array<string> r;
+  r= tm_string_split_at_spaces (s);
+  if (N(r) > 1) return r;
+  r= tm_string_split_between_words (s);
+  if (N(r) > 1) return r;
+  /* else split anywhere */
+  int i= 0, n= N(s);
+  while (i < n/2)
+    tm_char_forwards (s, i);
+  return array<string> (s(0, i), s(i, n));
+}
+
 /******************************************************************************
 * Quoting
 ******************************************************************************/
@@ -829,7 +946,7 @@ raw_quote (string s) {
   // Mark the label of a STRING tree as representing a string and not a symbol.
   return "\"" * s * "\"";
 }
- 
+
 string
 raw_unquote (string s) {
   // Get the string value of a STRING tree label representing a string.
@@ -851,6 +968,8 @@ escape_sh (string s) {
   string r;
   for (i=0; i<n; i++)
     switch (s[i]) {
+    case '(':
+    case ')':
     case '<':
     case '>':
     case '?':
@@ -1073,6 +1192,12 @@ skip_spaces (string s, int& i) {
   while ((i<n) && ((s[i]==' ') || (s[i]=='\t'))) i++;
 }
 
+void
+skip_whitespace (string s, int& i) {
+  int n=N(s);
+  while ((i<n) && ((s[i]==' ') || (s[i]=='\t') || (s[i]=='\n'))) i++;
+}
+
 void
 skip_line (string s, int& i) {
   int n=N(s);
@@ -1092,6 +1217,26 @@ skip_symbol (string s, int& i) {
   }
 }
 
+string
+convert_tabs_to_spaces (string s, int tw) {
+  int i= 0, ts= 0, n= N(s);
+  string r= "";
+  while (i<n) {
+    if (s[i] == '\t') {
+      r << string (' ', tw - ((i - ts) % tw));
+      ts= i+1;
+    }
+    else if (s[i] == '\n') {
+      ts= i+1;
+      r << s[i];
+    }
+    else
+      r << s[i];
+    i++;
+  }
+  return r;
+}
+
 /******************************************************************************
 * Parsing binary data
 ******************************************************************************/
@@ -1190,7 +1335,7 @@ search_backwards (string s, string in) {
 int
 count_occurrences (string s, string in) {
   int count= 0;
-  int i=0, next, n= N(s);
+  int i=0, next, n= N(in);
   while (i<n) {
     next= search_forwards (s, i, in);
     if (next == -1) break;
@@ -1261,11 +1406,22 @@ recompose (array<string> a, string sep) {
 }
 
 string
-trim_spaces (string s) {
-  int start, end;
+trim_spaces_left (string s) {
+  int start;
   for (start=0; start<N(s) && is_space (s[start]); start++) ;
-  for (end=N(s)-1; end > start && is_space (s[end]); end--) ;
-  return s (start, end+1);
+  return s (start, N(s));
+}
+
+string
+trim_spaces_right (string s) {
+  int end;
+  for (end=N(s)-1; end >= 0 && is_space (s[end]); end--) ;
+  return s (0, end+1);
+}
+
+string
+trim_spaces (string s) {
+  return trim_spaces_left (trim_spaces_right (s));
 }
 
 array<string>
@@ -1277,19 +1433,131 @@ trim_spaces (array<string> a) {
 }
 
 tree
-trim_spaces (tree t) {
-  if (is_atomic (t)) return trim_spaces (as_string (t));
+trim_spaces_right (tree t) {
+  if (is_atomic (t)) return trim_spaces_right (as_string (t));
   else if (is_concat (t)) {
-    int start, end;
-    for (start=0; start < N(t) && t[start] == " "; start++);
-    for (end=N(t)-1; end > start && t[end] == " "; end--);
+    tree l;
+    int end;
+    for (end= N(t)-1; end >= 0; end--) {
+      l= trim_spaces_right (t[end]);
+      if (l != "") break;
+    }
     tree r= tree (L(t));
-    for (int i=start; i<=end; i++) r << t[i];
-    return r;
+    for (int i=0; i<end; i++) r << t[i];
+    if (end >= 0) r << l;
+    if (N(r) == 0) return "";
+    else if (N(r) == 1) return r[0];
+    else return r;
   }
   else return t;
 }
 
+tree
+trim_spaces_left (tree t) {
+  if (is_atomic (t)) return trim_spaces_left (as_string (t));
+  else if (is_concat (t)) {
+    tree l;
+    int start;
+    for (start= 0; start < N(t); start++) {
+      l= trim_spaces_left (t[start]);
+      if (l != "") break;
+    }
+    tree r= tree (L(t));
+    if (start < N(t)) r << l;
+    for (int i=start+1; i<N(t); i++) r << t[i];
+    if (N(r) == 0) return "";
+    else if (N(r) == 1) return r[0];
+    else return r;
+  }
+  else return t;
+}
+
+tree
+trim_spaces (tree t) {
+  return trim_spaces_left (trim_spaces_right (t));
+}
+
+/******************************************************************************
+ * Program bracket matching
+ ******************************************************************************/
+
+extern tree the_et;
+enum dir_t { BACKWARD = -1, FORWARD = 1 }; // don't change the values
+
+  // FIXME: UGLY and won't handle some things
+static bool
+find_bracket_valid (tree t, int pos) {
+  if (pos < 0 || pos >= N(t))
+    return false;
+  if (L(t) == STRING || L(t) == DOCUMENT)
+    return true;
+  if (L(t) == CONCAT)
+    return L(t[pos]) == STRING || L(t[pos]) == CONCAT || L(t[pos]) == WITH;
+  if (L(t) == WITH)
+    return pos == N(t)-1 &&
+           (L(t[pos]) == STRING || L(t[pos]) == CONCAT || L(t[pos]) == WITH ||
+            L(t[pos]) == DOCUMENT);
+  else
+    return false;
+}
+
+static path
+find_bracket_sub (tree t, dir_t dir, const string& lbr, const string& rbr,
+                  int cnt, int pos) {
+  if (pos >= 0 && pos < N(t)) {
+    if (is_atomic (t)) {
+      void (*next) (string, int&) = (dir == FORWARD) ? tm_char_forwards
+                                                      : tm_char_backwards;
+      while (true) {
+        if (test (t->label, pos, lbr))
+          --cnt;
+        else if (test (t->label, pos, rbr))
+          ++cnt;
+        if (cnt == 0)
+          return reverse (path (min (N(t), pos), obtain_ip (t)));
+        int prev = pos;
+        next (t->label, pos);
+        if (prev == pos)
+          break;
+      }
+    } else if (find_bracket_valid (t, pos)) { // traverse child: t[pos]
+      int npos = (dir == FORWARD) ? 0 : N(t[pos])-1;
+      return find_bracket_sub (t[pos], dir, lbr, rbr, cnt, npos);
+    } else {  // traverse siblings
+      int npos = pos + (int)dir;
+      if (npos >= 0 && npos < N(t))
+        return find_bracket_sub (t, dir, lbr, rbr, cnt, npos);
+    }
+  }
+  path ip = obtain_ip (t);
+  if (is_nil (ip) || last_item (ip) < 0) return path();
+  const tree& pt = subtree (the_et, reverse (ip->next));
+  int npos = ip->item + (int)dir;
+  return find_bracket_sub (pt, dir, lbr, rbr, cnt, npos);
+}
+
+path
+find_left_bracket (path p, const string& lbr, const string& rbr) {
+  if (N(lbr) == 0 || N(rbr) == 0 || is_nil (p)) return path();
+  const tree& pt = subtree (the_et, path_up (p));
+  int pos = reverse(p)->item;
+  string s = as_string (pt);
+  if (pos < 0 || pos > N(s)) return path();
+  int cnt = test (s, pos, rbr) ? 0 : 1;
+  return find_bracket_sub (pt, BACKWARD, lbr, rbr, cnt, pos);
+}
+
+path
+find_right_bracket (path p, const string& lbr, const string& rbr) {
+  if (N(lbr) == 0 || N(rbr) == 0 || is_nil (p)) return path();
+  const tree& pt = subtree (the_et, path_up (p));
+  int pos = reverse(p)->item;
+  string s = as_string (pt);
+  if (pos < 0 || pos > N(s)) return path();
+  int cnt = test (s, pos, lbr) ? 0 : -1;
+  return find_bracket_sub (pt, FORWARD, lbr, rbr, cnt, pos);
+}
+
 /******************************************************************************
 * Computations with completions
 ******************************************************************************/
