@@ -161,6 +161,12 @@ converter_rep::load () {
   else if (from=="UTF-8" && to=="LaTeX" ) {
     hashtree<char,string> dic;
     hashtree_from_dictionary (dic,"utf8tolatex", UTF8, BIT2BIT, false);
+    hashtree_from_dictionary (dic,"utf8tolatex-onedir", UTF8, BIT2BIT, false);
+    ht = dic;
+  }
+  else if (from=="LaTeX" && to=="UTF-8" ) {
+    hashtree<char,string> dic;
+    hashtree_from_dictionary (dic,"utf8tolatex", BIT2BIT, UTF8, true);
     ht = dic;
   }
   else if (from=="Cork" && to=="ASCII") {
@@ -175,7 +181,7 @@ converter_rep::load () {
     hashtree_from_dictionary (dic,"tmuniversaltounicode", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"symbol-unicode-oneway", BIT2BIT, UTF8, false);
     hashtree_from_dictionary (dic,"symbol-unicode-math", BIT2BIT, UTF8, false);
-    hashtree_from_dictionary (dic,"cork-to-real-ascii", BIT2BIT, UTF8, false);
+    hashtree_from_dictionary (dic,"cork-to-real-ascii", BIT2BIT, BIT2BIT, false);
     ht = dic;
   }
   else if (from=="SourceCode" && to=="Cork") {
@@ -210,6 +216,8 @@ convert (string input, string from, string to) {
     return convert_from_cork (input, to);
   else if (to == "Cork")
     return convert_to_cork (input,from);
+  else if (from == "LaTeX" && to == "UTF-8")
+    return convert_LaTeX_to_utf8 (input);
   else if (from == "UTF-8" && to == "LaTeX")
     return convert_utf8_to_LaTeX (input);
 #ifdef USE_ICONV
@@ -248,6 +256,14 @@ convert_from_cork (string input, string to) {
 #endif
 }
 
+string
+convert_LaTeX_to_utf8 (string input) {
+  converter conv= load_converter ("LaTeX", "UTF-8");
+  string output;
+  output= apply (conv, input);
+  return output;
+}
+
 string
 convert_utf8_to_LaTeX (string input) {
   converter conv= load_converter ("UTF-8", "LaTeX");
@@ -453,13 +469,14 @@ iconv_converter::iconv_converter (string from2, string to2, bool errors):
   c_string to_cp   (to);
   cd= iconv_open (to_cp, from_cp);
   if (!is_valid() && show_errors)
-    system_error ("Initialization of iconv from " * from *
-		  " to " * to * " failed!");
+    convert_error << "Initialization of iconv from " << from
+                  << " to " << to << " failed\n";
   successful= true;
 }
 
 iconv_converter::~iconv_converter () {
-  iconv_close(cd);
+  if (is_valid())
+    iconv_close(cd);
 }
 
 // From the standard C++ library (remember, TeXmacs does _not_ use std!)
@@ -475,7 +492,7 @@ iconv_adaptor(size_t(*iconv_func)(iconv_t, T, size_t *, char**, size_t*),
 string apply (iconv_converter &conv, string input) {
   if (! conv.is_valid()) {
     conv.successful= false;
-    system_error ("Conversion concelled.");
+    convert_error << "Conversion cancelled\n";
     return input;
   }
   string result;
@@ -492,8 +509,8 @@ string apply (iconv_converter &conv, string input) {
 			     &in_cursor, &in_left, &out_cursor, &out_left);
     if(r == (size_t)-1 && errno != E2BIG) {
       if (conv.show_errors) {
-	cerr << "\nConverting from " << conv.from << " to " << conv.to << "\n";
-	system_error ("String conversion using iconv failed!");
+	convert_error << "Iconv conversion from " << conv.from
+                      << " to " << conv.to << " failed\n";
       }
       conv.successful= false;
       return input;
@@ -547,7 +564,7 @@ put_prefix_code (string key, string value, hashtree<char,string> tree) {
   if (DEBUG_STD) {
     hashtree<char,string> ht= find_node (key,tree);
     if (ht->label != "")
-      cout << "overwriting: " << ht->label << " with " << value << '\n';
+      debug_convert << "overwriting " << ht->label << " with " << value << '\n';
   }
   */
   find_node (key,tree)->set_label(value);
@@ -574,16 +591,17 @@ hashtree_from_dictionary (
   hashtree<char,string> dic, string file_name, escape_type key_escape,
   escape_type val_escape, bool reverse)
 {
-  system_info ("Loading",file_name);
+  if (DEBUG_STD) debug_convert << "Loading dictionary " << file_name << LF;
   string key_string, val_string, file;
   file_name = file_name * ".scm";
-  if (load_string (url ("$TEXMACS_PATH/langs/encoding", file_name), file, false)) {
-    system_error ("Couldn't open encoding dictionary", file_name);
+  if (load_string (url ("$TEXMACS_PATH/langs/encoding", file_name),
+                   file, false)) {
+    convert_error << "Couldn't open encoding dictionary " << file_name << LF;
     return;
   }
   tree t = block_to_scheme_tree (file);
   if (!is_tuple (t)) {
-    system_error ("Malformed encoding dictionary", file_name);
+    convert_error << "Malformed encoding dictionary " << file_name << LF;
     return;
   }
   for (int i=0; i<N(t); i++) {
@@ -828,3 +846,16 @@ utf8_to_hex_entities (string s) {
   }
   return result;
 }
+
+string
+utf8_to_hex_string (string s) {
+  string result;
+  int i, n= N(s);
+  for (i=0; i<n; ) {
+    unsigned int code= decode_from_utf8 (s, i);
+    string hex= as_hexadecimal (code);
+    while (N(hex) < 4) hex = "0" * hex;
+    result << hex;
+  }
+  return result;
+}
