@@ -17,6 +17,8 @@
 #include "timer.hpp"
 #include "Bridge/impl_typesetter.hpp"
 #include "new_style.hpp"
+#include "iterator.hpp"
+#include "merge_sort.hpp"
 #ifdef EXPERIMENTAL
 #include "../../Style/Environment/std_environment.hpp"
 #endif // EXPERIMENTAL
@@ -217,8 +219,7 @@ edit_typeset_rep::typeset_exec_until (path p) {
   //time_t t1= texmacs_time ();
   if (has_changed (THE_TREE + THE_ENVIRONMENT))
     if (p != correct_cursor (et, rp * 0)) {
-      if (DEBUG_STD)
-	cout << "TeXmacs] Warning: resynchronizing for path " << p << "\n";
+      if (DEBUG_STD) std_warning << "resynchronizing for path " << p << "\n";
       // apply_changes ();
     }
   if (p == tp && inside_graphics (true) && p != closest_inside (et, p)) {
@@ -227,13 +228,14 @@ edit_typeset_rep::typeset_exec_until (path p) {
     p = tp;
   }
 
+  //cout << "Exec until " << p << LF;
   if (N(cur[p])!=0) return;
   if (N(cur)>=25) // avoids out of memory in weird cases
     typeset_invalidate_env ();
   typeset_prepare ();
   if (enable_fastenv) {
     if (!(rp < p)) {
-      cerr << "TeXmacs] erroneous path " << p << "\n";
+      failed_error << "Erroneous path " << p << "\n";
       FAILED ("invalid typesetting path");
     }
     tree t= subtree (et, rp);
@@ -434,6 +436,7 @@ edit_typeset_rep::exec_texmacs (tree t) {
 
 tree
 edit_typeset_rep::exec_verbatim (tree t, path p) {
+  t= convert_OTS1_symbols_to_universal_encoding (t);
   typeset_exec_until (p);
   hashmap<string,tree> H= copy (cur[p]);
   H ("TeXmacs")= tree (MACRO, "TeXmacs");
@@ -449,6 +452,7 @@ edit_typeset_rep::exec_verbatim (tree t) {
 
 tree
 edit_typeset_rep::exec_html (tree t, path p) {
+  t= convert_OTS1_symbols_to_universal_encoding (t);
   if (p == (rp * 0)) typeset_preamble ();
   typeset_exec_until (p);
   hashmap<string,tree> H= copy (cur[p]);
@@ -498,12 +502,16 @@ value_to_compound (tree t, hashmap<string,tree> h) {
 
 tree
 edit_typeset_rep::exec_latex (tree t, path p) {
+  t= convert_OTS1_symbols_to_universal_encoding (t);
   string pref= "texmacs->latex:expand-macros";
   if (as_string (call ("get-preference", pref)) != "on") return t;
   if (p == (rp * 0)) typeset_preamble ();
   typeset_exec_until (p);
   hashmap<string,tree> H= copy (cur[p]);
-  tree patch= as_tree (call ("stree->tree", call ("tmtex-env-patch", t)));
+  object l= null_object ();
+  iterator<string> it= iterate (H);
+  while (it->busy ()) l= cons (object (it->next ()), l);
+  tree patch= as_tree (call ("stree->tree", call ("tmtex-env-patch", t, l)));
   hashmap<string,tree> P (UNINIT, patch);
   H->join (P);
   if (is_document (t) && is_compound (t[0], "hide-preamble")) {
@@ -584,19 +592,76 @@ edit_typeset_rep::init_default (string var) {
 ******************************************************************************/
 
 void
-edit_typeset_rep::typeset (SI& x1, SI& y1, SI& x2, SI& y2) {
+edit_typeset_rep::typeset_sub (SI& x1, SI& y1, SI& x2, SI& y2) {
   //time_t t1= texmacs_time ();
   typeset_prepare ();
   eb= empty_box (reverse (rp));
   // saves memory, also necessary for change_log update
   bench_start ("typeset");
-  eb= ::typeset (ttt, x1, y1, x2, y2);
+#ifdef USE_EXCEPTIONS
+  try {
+#endif
+    eb= ::typeset (ttt, x1, y1, x2, y2);
+#ifdef USE_EXCEPTIONS
+  }
+  catch (string msg) {
+    the_exception= msg;
+    std_error << "Typesetting failure, resetting to empty document\n";
+    assign (rp, tree (DOCUMENT, ""));
+    ::notify_assign (ttt, path(), subtree (et, rp));
+    eb= ::typeset (ttt, x1, y1, x2, y2);    
+  }
+  handle_exceptions ();
+#endif
   bench_end ("typeset");
   //time_t t2= texmacs_time ();
   //if (t2 - t1 >= 10) cout << "typeset took " << t2-t1 << "ms\n";
   picture_cache_clean ();
 }
 
+static void
+report_missing (hashmap<string,tree> missing) {
+  array<string> a;
+  for (iterator<string> it= iterate (missing); it->busy(); a << it->next ());
+  merge_sort (a);
+  for (int i=0; i<N(a); i++)
+    if (!starts (a[i], "bib-"))
+      typeset_warning << "Undefined reference " << a[i] << LF;
+}
+
+static void
+report_redefined (array<tree> redefined) {
+  for (int i=0; i<N(redefined); i++) {
+    tree t= redefined[i];
+    if (t[1]->label == "")
+      typeset_warning << "Redefined " << t[0]->label << LF;
+    else
+      typeset_warning << "Redefined " << t[0]->label
+                      << " as " << t[1]->label << LF;
+  }
+}
+
+void
+edit_typeset_rep::typeset (SI& x1, SI& y1, SI& x2, SI& y2) {
+  int missing_nr= 1000000;
+  int redefined_nr= 1000000;
+  while (true) {
+    typeset_sub (x1, y1, x2, y2);
+    if (!env->complete) break;
+    env->complete= false;
+    if (N(env->missing) == 0 && N(env->redefined) == 0) break;
+    if ((N(env->missing) == missing_nr && N(env->redefined) == redefined_nr) ||
+        (N(env->missing) > missing_nr || N(env->redefined) > redefined_nr)) {
+      report_missing (env->missing);
+      report_redefined (env->redefined);
+      break;
+    }
+    missing_nr= N(env->missing);
+    redefined_nr= N(env->redefined);
+    ::notify_assign (ttt, path(), ttt->br->st);
+  }
+}
+
 void
 edit_typeset_rep::typeset_invalidate (path p) {
   if (rp <= p) {
