@@ -33,8 +33,7 @@ MODE_LANGUAGE (string mode) {
   else if (mode == "math") return MATH_LANGUAGE;
   else if (mode == "prog") return PROG_LANGUAGE;
   else if (mode == "src") return LANGUAGE;
-  cerr << "Mode = " << mode << "\n";
-  FAILED ("invalid mode");
+  std_error << "Invalid mode " << mode << ", assuming text mode instead\n";
   return LANGUAGE;
 }
 
@@ -54,7 +53,7 @@ edit_interface_rep::edit_interface_rep ():
   magf (zoomf / std_shrinkf),
   pixel ((SI) tm_round ((std_shrinkf * PIXEL) / zoomf)), copy_always (),
   last_x (0), last_y (0), last_t (0),
-  made_selection (false), table_selection (false), mouse_adjusting(false),
+  table_selection (false), mouse_adjusting (false),
   oc (0, 0), temp_invalid_cursor (false),
   shadow (NULL), stored (NULL),
   cur_sb (2), cur_wb (2)
@@ -98,9 +97,23 @@ edit_interface_rep::resume () {
   SERVER (menu_icons (3, "(horizontal (link texmacs-extra-icons))"));
   if (use_side_tools)
     { SERVER (side_tools (0, "(vertical (link texmacs-side-tools))")); }
+  SERVER (bottom_tools (0, "(vertical (link texmacs-bottom-tools))"));
   cur_sb= 2;
-  tp= make_cursor_accessible (tp, true);
-  notify_change (THE_FOCUS + THE_EXTENTS + THE_CURSOR);
+  notify_change (THE_FOCUS + THE_EXTENTS);
+  path new_tp= make_cursor_accessible (tp, true);
+  if (new_tp != tp) {
+    notify_change (THE_CURSOR);
+    tp= new_tp;
+  }
+}
+
+void
+edit_interface_rep::keyboard_focus_on (string field) {
+  array<url> a= buffer_to_windows (buf->buf->name);
+  if (N(a) >= 1) {
+    tm_window win= concrete_window (a[0]);
+    send_keyboard_focus_on (win->wid, field);
+  }
 }
 
 /******************************************************************************
@@ -450,13 +463,9 @@ edit_interface_rep::apply_changes () {
       SERVER (menu_icons (3, "(horizontal (link texmacs-extra-icons))"));
       if (use_side_tools)
         { SERVER (side_tools (0, "(vertical (link texmacs-side-tools))")); }
+      SERVER (bottom_tools (0, "(vertical (link texmacs-bottom-tools))"));
       set_footer ();
-      if (has_current_window ()) {
-        if (need_save())
-          concrete_window () -> set_window_name (buf->buf->title * " *");
-        else
-          concrete_window () -> set_window_name (buf->buf->title);
-      }
+      if (has_current_window ()) concrete_window()->set_modified (need_save());
       if (!gui_interrupted ()) drd_update ();
       cache_memorize ();
       last_update= last_change;
@@ -523,14 +532,21 @@ edit_interface_rep::apply_changes () {
   
   // cout << "Handling selection\n";
   if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_SELECTION)) {
-    if (made_selection) {
+    if (!is_nil (selection_rects)) {
       invalidate (selection_rects);
       if (!selection_active_any ()) {
-        made_selection= false;
         set_selection (tp, tp);
         selection_rects= rectangles ();
       }
     }
+    if (N (alt_selection_rects) != 0) {
+      rectangles visible (rectangle (vx1, vy1, vx2, vy2));
+      for (int i=0; i<N(alt_selection_rects); i++)
+        invalidate (alt_selection_rects[i] & visible);
+      range_set alt_sel= get_alt_selection ("alternate");
+      if (is_empty (alt_sel))
+        alt_selection_rects= array<rectangles> ();
+    }
   }
   
   // cout << "Handling environment\n";
@@ -563,10 +579,10 @@ edit_interface_rep::apply_changes () {
   // cout << "Handling extents\n";
   if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS)) {
     string medium= get_init_string (PAGE_MEDIUM);
-    SI ex1= (SI) eb->x1*magf;
-    SI ey1= (SI) eb->y1*magf;
-    SI ex2= (SI) eb->x2*magf;
-    SI ey2= (SI) eb->y2*magf;
+    SI ex1= (SI) (((double) eb->x1) * magf);
+    SI ey1= (SI) (((double) eb->y1) * magf);
+    SI ex2= (SI) (((double) eb->x2) * magf);
+    SI ey2= (SI) (((double) eb->y2) * magf);
     abs_round (ex1, ey1);
     abs_round (ex2, ey2);
     SI w, h;
@@ -692,8 +708,7 @@ edit_interface_rep::apply_changes () {
   
   // cout << "Handling selection\n";
   if (env_change & THE_SELECTION) {
-    made_selection= selection_active_any ();
-    if (made_selection) {
+    if (selection_active_any ()) {
       table_selection= selection_active_table ();
       selection sel; selection_get (sel);
       rectangles rs= thicken (sel->rs, pixel, 3*pixel);
@@ -703,11 +718,29 @@ edit_interface_rep::apply_changes () {
       selection_rects= rs;
       invalidate (selection_rects);
     }
+    range_set alt_sel= get_alt_selection ("alternate");
+    if (!is_empty (alt_sel)) {
+      alt_selection_rects= array<rectangles> ();
+      for (int i=0; i+1<N(alt_sel); i+=2) {
+        range_set sub_sel= simple_range (alt_sel[i], alt_sel[i+1]);
+        selection sel= compute_selection (sub_sel);
+        rectangles rs= thicken (sel->rs, pixel, 3*pixel);
+#ifndef QTTEXMACS
+        rs= simplify (::correct (rs - thicken (rs, -pixel, -pixel)));
+#endif
+        if (N(rs) != 0) alt_selection_rects << rs;
+      }
+      rectangles visible (rectangle (vx1, vy1, vx2, vy2));
+      for (int i=0; i<N(alt_selection_rects); i++)
+        invalidate (alt_selection_rects[i] & visible);
+    }
   }
   
   // cout << "Handling locus highlighting\n";
-  if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS))
-    update_active_loci ();
+  if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS)) {
+    update_mouse_loci ();
+    update_focus_loci ();
+  }
   if (env_change & THE_LOCUS) {
     if (locus_new_rects != locus_rects) {
       invalidate (locus_rects);
@@ -786,6 +819,13 @@ edit_interface_rep::after_menu_action () {
   windows_delayed_refresh (1);
 }
 
+void
+edit_interface_rep::cancel_menu_action () {
+  notify_change (THE_DECORATIONS);
+  cancel_editing ();
+  windows_delayed_refresh (1);
+}
+
 rectangle
 edit_interface_rep::get_window_extents () {
   SI ox, oy, w, h;
