@@ -12,6 +12,7 @@
 #ifndef EDITOR_H
 #define EDITOR_H
 #include "typesetter.hpp"
+#include "tree_select.hpp"
 #ifdef AQUATEXMACS
 #  include "Cocoa/aqua_simple_widget.h"
 #else
@@ -135,6 +136,7 @@ public:
   /* public routines from edit_interface */
   virtual void suspend () = 0;
   virtual void resume () = 0;
+  virtual void keyboard_focus_on (string field) = 0;
   virtual int  get_pixel_size () = 0;
   virtual SI   get_visible_width () = 0;
   virtual SI   get_visible_height () = 0;
@@ -150,6 +152,7 @@ public:
   virtual void full_screen_mode (bool flag) = 0;
   virtual void before_menu_action () = 0;
   virtual void after_menu_action () = 0;
+  virtual void cancel_menu_action () = 0;
   virtual rectangle get_window_extents () = 0;
   virtual cursor search_cursor (path p) = 0;
   virtual selection search_selection (path start, path end) = 0;
@@ -175,10 +178,11 @@ public:
   virtual void mouse_drag (SI x, SI y) = 0;
   virtual void mouse_select (SI x, SI y, int mods, bool drag) = 0;
   virtual void mouse_paste (SI x, SI y) = 0;
-  virtual void mouse_adjust (SI x, SI y) = 0;
+  virtual void mouse_adjust (SI x, SI y, int mods) = 0;
   virtual void mouse_adjust_selection (SI x, SI y, int mods) = 0;
   virtual void mouse_scroll (SI x, SI y, bool up) = 0;
   virtual cursor get_cursor () = 0;
+  virtual array<SI> get_mouse_position () = 0;
   virtual void set_pointer (string name) = 0;
   virtual void set_pointer (string curs_name, string mask_name) = 0;
   virtual void set_message (tree l, tree r= "", bool temp= false) = 0;
@@ -302,6 +306,7 @@ public:
   virtual void archive_state () = 0;
   virtual void start_editing () = 0;
   virtual void end_editing () = 0;
+  virtual void cancel_editing () = 0;
   virtual void start_slave (double a) = 0;
   virtual void mark_start (double a) = 0;
   virtual bool mark_cancel (double a) = 0;
@@ -452,6 +457,8 @@ public:
   virtual tree selection_raw_get (string key) = 0;
   virtual void selection_correct (path i1, path i2, path& o1, path& o2) = 0;
   virtual path selection_get_subtable (int& i1, int& j1, int& i2, int& j2) = 0;
+  virtual selection compute_selection (path p1, path p2) = 0;
+  virtual selection compute_selection (range_set sel) = 0;
   virtual void selection_get (selection& sel) = 0;
   virtual void selection_get (path& start, path& end) = 0;
   virtual path selection_get_start () = 0;
@@ -464,6 +471,7 @@ public:
   virtual void selection_set_start (path p= path()) = 0;
   virtual void selection_set_end (path p= path()) = 0;
   virtual void selection_set_paths (path start, path end) = 0;
+  virtual void selection_set_range_set (range_set sel) = 0;
   virtual void selection_copy (string key= "primary") = 0;
   virtual void selection_paste (string key= "primary") = 0;
   virtual void selection_clear (string key= "primary") = 0;
@@ -484,6 +492,11 @@ public:
   virtual void manual_focus_release () = 0;
   virtual path focus_get (bool skip_flag= true) = 0;
 
+  virtual void set_alt_selection (string s, range_set sel) = 0;
+  virtual range_set get_alt_selection (string s) = 0;
+  virtual void cancel_alt_selection (string s) = 0;
+  virtual void cancel_alt_selections () = 0;
+
   /* public routines from edit_replace */
   virtual bool inside (string what) = 0;
   virtual bool inside (tree_label l) = 0;
@@ -516,6 +529,7 @@ public:
   virtual url  get_name () = 0;
   virtual void focus_on_this_editor () = 0;
   virtual void notify_page_change () = 0;
+  virtual string get_metadata (string kind) = 0;
   virtual void print (url ps_name, bool to_file, int first, int last) = 0;
   virtual void print_to_file (url ps_name,
 			      string first="1", string last="1000000") = 0;
