@@ -629,7 +629,7 @@ smart_font_rep::resolve (string c, string fam, int attempt) {
       if (fn[nr]->supports (rewrite (c, REWRITE_MATH)))
         return sm->add_char (key, c);
     }
-    if (fam == "roman") {
+    if (fam == "roman" && N(c) > 1) {
       tree key= tuple ("cyrillic", fam, variant, series, rshape);
       int nr= sm->add_font (key, REWRITE_CYRILLIC);
       initialize_font (nr);
@@ -741,8 +741,9 @@ smart_font_rep::initialize_font (int nr) {
   }
   //cout << "Font " << nr << ", " << a << " -> " << fn[nr]->res_name << "\n";
   if (fn[nr]->res_name == res_name) {
-    cout << "Font " << nr << ", " << a << " -> " << fn[nr]->res_name << "\n";
-    ASSERT (false, "substitution font loop detected");
+    failed_error << "Font " << nr << ", " << a
+                 << " -> " << fn[nr]->res_name << "\n";
+    FAILED ("substitution font loop detected");
   }
 }
 
@@ -791,13 +792,17 @@ smart_font_rep::get_extents (string s, metric& ex) {
     int nr;
     string r= s;
     metric ey;
-    while (i < n) {
+    while (true) {
       advance (s, i, r, nr);
       if (nr >= 0) {
         //cout << "From " << nr << " -> " << sm->fn_spec[nr] << "\n";
         fn[nr]->get_extents (r, ex);
         break;
       }
+      if (i >= n) {
+        fn[0]->get_extents (empty_string, ex);
+        break;
+      }
     }
     while (i < n) {
       advance (s, i, r, nr);
@@ -820,18 +825,28 @@ void
 smart_font_rep::get_xpositions (string s, SI* xpos) {
   SI x= 0;
   int i=0, n= N(s);
+  xpos[0]= x;
   while (i < n) {
     int nr;
     string r= s;
     int start= i;
     advance (s, i, r, nr);
     if (nr >= 0) {
-      fn[nr]->get_xpositions (r, xpos+start);
-      for (int j=0; j<=N(r); j++) xpos[start+j] += x;
+      if (r == s (start, i)) {
+        fn[nr]->get_xpositions (r, xpos+start);
+        for (int j=0; j<=N(r); j++) xpos[start+j] += x;
+      }
+      else {
+        STACK_NEW_ARRAY (tmp, SI, N(r)+1);
+        fn[nr]->get_xpositions (r, tmp);
+        for (int j=start; j<i; j++) xpos[j]= x;
+        xpos[i]= x + tmp[N(r)];
+        STACK_DELETE_ARRAY (tmp);
+      }
       x= xpos[i];
     }
     else
-      for (int j=0; j<=N(r); j++) xpos[start+j]= x;
+      for (int j=start; j<=i; j++) xpos[j]= x;
   }
 }
 
@@ -839,18 +854,28 @@ void
 smart_font_rep::get_xpositions (string s, SI* xpos, SI xk) {
   SI x= 0;
   int i=0, n= N(s);
+  xpos[0]= x;
   while (i < n) {
     int nr;
     string r= s;
     int start= i;
     advance (s, i, r, nr);
     if (nr >= 0) {
-      fn[nr]->get_xpositions (r, xpos+start, xk);
-      for (int j=0; j<=N(r); j++) xpos[start+j] += x;
+      if (r == s (start, i)) {
+        fn[nr]->get_xpositions (r, xpos+start, xk);
+        for (int j=0; j<=N(r); j++) xpos[start+j] += x;
+      }
+      else {
+        STACK_NEW_ARRAY (tmp, SI, N(r)+1);
+        fn[nr]->get_xpositions (r, tmp, xk);
+        for (int j=start; j<i; j++) xpos[j]= x;
+        xpos[i]= x + tmp[N(r)];
+        STACK_DELETE_ARRAY (tmp);
+      }
       x= xpos[i];
     }
     else
-      for (int j=0; j<=N(r); j++) xpos[start+j]= x;
+      for (int j=start; j<=i; j++) xpos[j]= x;
   }
 }
 
@@ -961,12 +986,33 @@ smart_font (string family, string variant, string series, string shape,
   if (starts (family, "tc"))
     // FIXME: temporary hack for symbols from std-symbol.ts
     return find_font (family, variant, series, shape, sz, dpi);
+  if (starts (family, "sys-")) {
+    if (family == "sys-chinese") {
+      string name= default_chinese_font_name ();
+      family= "cjk=" * name * ",roman";
+    }
+    if (family == "sys-japanese") {
+      string name= default_japanese_font_name ();
+      family= "cjk=" * name * ",roman";
+    }
+    if (family == "sys-korean") {
+      string name= default_korean_font_name ();
+      family= "cjk=" * name * ",roman";
+    }
+  }
 
   string name=
     family * "-" * variant * "-" *
     series * "-" * shape * "-" *
     as_string (sz) * "-" * as_string (dpi) * "-smart";
   if (font::instances->contains (name)) return font (name);
+  if (starts (shape, "math")) {
+    array<string> a= trimmed_tokenize (family, ","), b;
+    for (int i=0; i<N(a); i++)
+      if (starts (a[i], "math=")) b << a[i] (5, N(a[i]));
+      else b << a[i];
+    family= recompose (b, ",");
+  }
   string sh= shape;
   if (shape == "mathitalic" || shape == "mathshape") sh= "right";
   string mfam= main_family (family);
