@@ -21,6 +21,7 @@
 #include "image_files.hpp"
 #include "link.hpp"
 #include "frame.hpp"
+#include "converter.hpp"
 
 string PS_CLIP_PUSH ("gsave");
 string PS_CLIP_POP ("grestore");
@@ -50,9 +51,9 @@ printer_rep::printer_rep (
     use_alpha (get_preference ("experimental alpha") == "on"),
     linelen (0), fg ((color) (-1)), bg ((color) (-1)), opacity (255),
     ncols (0), lw (-1), nwidths (0), cfn (""), nfonts (0),
-    xpos (0), ypos (0), tex_flag (false),
+    xpos (0), ypos (0), tex_flag (false), toc (TUPLE),
     defs ("?"), tex_chars ("?"), tex_width ("?"),
-    tex_fonts ("?"), tex_font_chars (array<int>(0))    
+    tex_fonts ("?"), tex_font_chars (array<int>(0)), metadata ("")
 {
   string tex_pro, special_pro, color_pro, texps_pro;
   load_string ("$TEXMACS_PATH/misc/convert/tex.pro", tex_pro, true);
@@ -120,6 +121,8 @@ printer_rep::printer_rep (
 
 printer_rep::~printer_rep () {
   next_page ();
+  generate_toc ();
+  generate_metadata ();
   body << "\n%%Trailer\n"
        << "end\n"
        << "userdict /end-hook known{end-hook} if\n"
@@ -934,14 +937,14 @@ printer_rep::draw_scalable (scalable im, SI x, SI y, int alpha) {
 }
 
 void
-printer_rep::anchor (string label, SI x, SI y) {
+printer_rep::anchor (string label, SI x1, SI y1, SI x2, SI y2) {
   string s = "(";
   s = s << prepare_text (label) << ") cvn";
   if (linelen>0) cr ();
   print ("[ /Dest");
   print (s);
   print ("/View [/XYZ");
-  print (x, y);
+  print (x1, y1);
   print ("null] /DEST pdfmark");
   cr ();
 }
@@ -974,9 +977,95 @@ printer_rep::href (string label, SI x1, SI y1, SI x2, SI y2) {
 
 void
 printer_rep::toc_entry (string kind, string title, SI x, SI y) {
-  // FIXME: not yet implemented
-  (void) kind; (void) title; (void) x; (void) y;
-  // cout << kind << ", " << title << "\n";
+  decode (x, y);
+  string ls= "1";
+  if (kind == "toc-strong-1") ls= "1";
+  if (kind == "toc-strong-2") ls= "2";
+  if (kind == "toc-1") ls= "3";
+  if (kind == "toc-2") ls= "4";
+  if (kind == "toc-3") ls= "5";
+  if (kind == "toc-4") ls= "6";
+  if (kind == "toc-5") ls= "7";
+  string ps= as_string (cur_page);
+  string xs= as_string (x-dpi);
+  string ys= as_string (y-dpi);
+  toc << tuple (title, ls, ps, xs, ys);
+}
+
+void
+structure_toc (tree t, int& i, tree& out, string level) {
+  //cout << "Structure " << t[i] << " at " << level << "\n";
+  if (i >= N(t)) return;
+  string nlevel= t[i][1]->label;
+  if (nlevel <= level) return;
+  tree next= tuple (t[i]);
+  i++;
+  while (i < N(t)) {
+    string slevel= t[i][1]->label;
+    if (slevel <= nlevel) break;
+    structure_toc (t, i, next, nlevel);
+  }
+  out << next;
+}
+
+static string
+to_hex_string (string s) {
+  return "<FEFF" * utf8_to_hex_string (cork_to_utf8 (s)) * ">";  
+}
+
+void
+printer_rep::generate_toc_item (tree t) {
+  string title= t[0][0]->label;
+  string level= t[0][1]->label;
+  string page = t[0][2]->label;
+  string x    = t[0][3]->label;
+  string y    = t[0][4]->label;
+  string htit = to_hex_string (title);
+  print ("[");
+  if (N(t) > 1) {
+    print ("/Count");
+    print (as_string (1 - N(t)));
+  }
+  print ("/Page");
+  print (page);
+  print ("/View [ /XYZ");
+  print (x);
+  print (y);
+  print ("0");
+  print ("]");
+  print ("/Title " * htit);
+  print ("/OUT pdfmark");
+  cr ();
+  for (int i=1; i<N(t); i++)
+    generate_toc_item (t[i]);
+}
+
+void
+printer_rep::generate_toc () {
+  tree rew (TUPLE);
+  int i=0;
+  while (i < N(toc))
+    structure_toc (toc, i, rew, "0");
+  for (i=0; i<N(rew); i++)
+    generate_toc_item (rew[i]);
+}
+
+void
+printer_rep::set_metadata (string kind, string val) {
+  metadata (kind)= val;
+}
+
+void
+printer_rep::generate_metadata () {
+  if (N(metadata) == 0) return;
+  print ("[");
+  if (metadata->contains ("title"))
+    print ("/Title " * to_hex_string (metadata ["title"]));
+  if (metadata->contains ("author"))
+    print ("/Author " * to_hex_string (metadata ["author"]));
+  if (metadata->contains ("subject"))
+    print ("/Subject " * to_hex_string (metadata ["subject"]));
+  print ("/DOCINFO pdfmark");
 }
 
 /******************************************************************************
