@@ -22,7 +22,8 @@ bool gs_supports (url image);
 void gs_image_size (url image, int& w_pt, int& h_pt);
 void gs_to_png (url image, url png, int w_px, int h_px);
 void gs_to_eps (url image, url eps);
-void gs_to_pdf (url doc, url pdf, bool landscape, double paper_h, double paper_w);
+void gs_to_pdf (url doc, url pdf, bool landsc, double paper_h, double paper_w);
+void gs_to_ps (url doc, url ps, bool landsc, double paper_h, double paper_w);
 void tm_gs (url image);
 
 #endif // USE_GS
