@@ -28,88 +28,14 @@
 #include "QTMGuiHelper.hpp"
 #include "QTMMenuHelper.hpp"
 #include "QTMStyle.hpp"
+#include "QTMApplication.hpp"
+#include "QTMTreeModel.hpp"
 
 #include <QCheckBox>
 #include <QPushButton>
 #include <QSplitter>
+#include <QTreeView>
 
-/******************************************************************************
- * Auxiliary classes
- ******************************************************************************/
-
-QTMMenuButton::QTMMenuButton (QWidget* parent) : QToolButton(parent) {
-  QTMAuxMenu m;
-  m.myInitStyleOption (&option);
-  setAttribute (Qt::WA_Hover);
-}
-
-void
-QTMMenuButton::mousePressEvent (QMouseEvent* event) {
-  // this one triggers the action and toggles the button
-  QToolButton::mousePressEvent (event);
-  // this one forwards the event to the parent
-  // (which eventually is the menu)
-  QWidget::mousePressEvent (event);
-}
-
-void
-QTMMenuButton::mouseReleaseEvent (QMouseEvent* event) {
-  // this one triggers the action and untoggles the button
-  QToolButton::mouseReleaseEvent (event);
-  // this one forwards the event to the parent
-  // (which eventually is the menu which then closes itself)
-  QWidget::mouseReleaseEvent (event);
-}
-
-void
-QTMMenuButton::paintEvent (QPaintEvent* event) {
-  (void) event;
-  QPainter p (this);
-  
-  // initialize the options
-  QStyleOptionToolButton buttonOpt;
-  initStyleOption (&buttonOpt);
-  QRect r = rect ();
-  option.rect = r;
-  option.state = QStyle::State_Enabled |
-  ( buttonOpt.state & QStyle::State_MouseOver ? 
-   QStyle::State_Selected : QStyle::State_None ); 
-  // draw the control background as a menu item
-  style () -> drawControl (QStyle::CE_MenuItem, &option, &p, this); 
-  // draw the icon with a bit of inset.
-  r.adjust (2,2,-2,-2);
-  defaultAction ()-> icon ().paint (&p, r);
-}
-
-QTMMenuWidget::QTMMenuWidget (QWidget* parent): QWidget(parent) {
-  QTMAuxMenu m;
-  m.myInitStyleOption (&option);
-}
-
-void
-QTMMenuWidget::paintEvent(QPaintEvent* event) {
-  (void) event;
-  QPainter p (this);
-  option.rect = rect ();
-  //QRect r = rect ();
-  style()->drawControl (QStyle::CE_MenuEmptyArea, &option, &p, this);
-  QWidget::paintEvent(event);
-}
-
-
-/*! A QToolButton which draws itself using its defaultAction()'s icon. */
-class QTMUIButton: public QToolButton {
-public:
-  QTMUIButton (QWidget* parent = NULL): QToolButton(parent) {}
-  void paintEvent(QPaintEvent *event);
-};
-
-void
-QTMUIButton::paintEvent(QPaintEvent* event) {
-  (void) event;
-  QPainter p (this);
-  defaultAction()->icon().paint (&p, rect ());
-}
 
 /******************************************************************************
  * Ad-hoc command_rep derivates for different UI elements in qt_ui_element_rep
@@ -130,7 +56,7 @@ public:
     if (N(ks)) { 
       QTMWidget *w = qobject_cast<QTMWidget*>(qApp->focusWidget());
       if (w && w->tm_widget()) {
-        if (DEBUG_QT) cout << "shortcut: " << ks << LF;
+        if (DEBUG_QT) debug_qt << "shortcut: " << ks << LF;
         the_gui->process_keypress (w->tm_widget(), ks, texmacs_time());
       }
     }
@@ -187,17 +113,17 @@ public:
 
 QPixmap 
 qt_glue_widget_rep::render () {
-  QSize s = to_qsize(w, h);
+  QSize s = to_qsize (w, h);
   QPixmap pxm(s);
     //cout << "glue (" << s.width() << "," << s.height() << ")\n";
   pxm.fill (Qt::transparent);
   QPaintDevice *pd = static_cast<QPaintDevice*>(&pxm);
   
   if (pd && !pxm.isNull()) {
-    qt_renderer_rep *ren = the_qt_renderer();
+    qt_renderer_rep* ren = the_qt_renderer();
     ren->begin (pd);
     rectangle r = rectangle (0, 0, s.width(), s.height());
-    ren->set_origin(0,0);
+    ren->set_origin (0,0);
     ren->encode (r->x1, r->y1);
     ren->encode (r->x2, r->y2);
     ren->set_clipping (r->x1, r->y2, r->x2, r->y1);
@@ -206,13 +132,13 @@ qt_glue_widget_rep::render () {
         // do nothing
     } else {
       if (is_atomic (col)) {
-        color c= named_color (col->label);
+        color c = named_color (col->label);
         ren->set_background (c);
         ren->set_pencil (c);
         ren->fill (r->x1, r->y2, r->x2, r->y1);
       } else {
         ren->set_shrinking_factor (std_shrinkf);
-        brush old_b= ren->get_background ();
+        brush old_b = ren->get_background ();
         ren->set_background (col);
         ren->clear_pattern (5*r->x1, 5*r->y2, 5*r->x2, 5*r->y1);
         ren->set_background (old_b);
@@ -227,7 +153,7 @@ qt_glue_widget_rep::render () {
 
 QAction *
 qt_glue_widget_rep::as_qaction() {
-  QAction *a= new QTMAction();
+  QAction* a = new QTMAction();
   a->setText (to_qstring (as_string (col)));
   QIcon icon;
 #if 0
@@ -246,11 +172,12 @@ qt_glue_widget_rep::as_qaction() {
 
 QWidget *
 qt_glue_widget_rep::as_qwidget() {
-  QLabel* w = new QLabel();
-  w->setText (to_qstring (as_string (col)));
-  w->setPixmap (render ());  
+  QLabel* qw = new QLabel();
+  qw->setText (to_qstring (as_string (col)));
+  qw->setPixmap (render ());
+  qw->setMinimumSize (to_qsize (w, h));
     //  w->setEnabled(false);
-  qwid = w;
+  qwid = qw;
   return qwid;
 }
 
@@ -260,34 +187,70 @@ qt_glue_widget_rep::as_qwidget() {
  ******************************************************************************/
 
 qt_ui_element_rep::qt_ui_element_rep (types _type, blackbox _load)
-  : qt_widget_rep(_type), load(_load), cachedAction(NULL)  { }
+: qt_widget_rep (_type), load (_load), cachedAction (NULL), cachedMenu (NULL) {}
 
+/*!
+ WARNING: Deleting here the cachedAction would contradict our memory policy
+ in qt_menu_rep and lead to crashes
+ 
+ HOWEVER: if we have a cachedMenu, it means somebody promoted us to managers
+ of a root menu which we have to delete. But we MUST ensure nobody calls
+ get_qmenu() when they shouldn't.
+ */
 qt_ui_element_rep::~qt_ui_element_rep() {
-  if (cachedAction) delete cachedAction;
+  delete cachedMenu;
+}
+
+blackbox
+qt_ui_element_rep::get_payload (qt_widget qtw, types check_type) {
+  ASSERT (check_type == none || qtw->type == check_type,
+          c_string ("get_payload: widget " * qtw->type_as_string() *
+                    " was not of the expected type."));
+  switch (qtw->type) {
+    case horizontal_menu:   case vertical_menu:    case horizontal_list:
+    case vertical_list:     case tile_menu:        case aligned_widget:
+    case minibar_menu:      case menu_separator:   case menu_group:
+    case pulldown_button:   case pullright_button: case menu_button:
+    case text_widget:       case xpm_widget:       case toggle_widget:
+    case enum_widget:       case choice_widget:    case filtered_choice_widget:
+    case scrollable_widget: case hsplit_widget:    case vsplit_widget:
+    case tabs_widget:       case icon_tabs_widget: case resize_widget:
+    case refresh_widget:    case refreshable_widget:  case balloon_widget:
+    case glue_widget:
+    {
+      qt_ui_element_rep* rep = static_cast<qt_ui_element_rep*> (qtw.rep);
+      return rep->load;
+    }
+      break;
+    default:
+      return blackbox();
+  }
 }
 
 /*! Returns the ui element as a popup widget.
  If the widget is of type vertical_menu, it is understood that the popup widget
- must be of the standard OS dependent type implemented by qt_menu_rep using 
+ must be of the standard OS dependent type implemented by qt_menu_rep using
  QMenu.
  */
 widget 
 qt_ui_element_rep::make_popup_widget () {
   if (type == qt_widget_rep::vertical_menu)
-    return tm_new<qt_menu_rep>(as_qaction());
+    return tm_new<qt_menu_rep> (this);
   else
     return qt_widget_rep::make_popup_widget();
 }
 
-QMenu *
+/*! Returns a QMenu* to be installed in the menu bar.
+ This method *DOES NOT* transfer ownership to the caller. On the contrary, it
+ "promotes" this qt_ui_element_rep to responsible for its deletion.
+ */
+QMenu*
 qt_ui_element_rep::get_qmenu() {
-  if (!cachedAction) {
-    cachedAction = as_qaction();
-  }
-  return (cachedAction ? cachedAction->menu() : NULL);
+  delete cachedMenu;
+  cachedMenu = as_qaction()->menu();
+  return cachedMenu;
 }
 
-
 /*! For the refresh_widget
  * FIXME? Is this really used?
  */
@@ -297,7 +260,7 @@ qt_ui_element_rep::operator tree () {
     T x= open_box<T> (load);
     return tree (TUPLE, "refresh", x.x1, x.x2);
   }
-  else if (type == refresh_widget) {
+  else if (type == refreshable_widget) {
     typedef pair<object, string> T;
     T x= open_box<T> (load);
     return tree (TUPLE, "refreshable", x.x2);
@@ -310,7 +273,11 @@ qt_ui_element_rep::operator tree () {
 QAction* 
 qt_ui_element_rep::as_qaction () {
   //if (DEBUG_QT_WIDGETS)
-  //cout << "as_qaction: " << type_as_string() << LF;
+  //debug_widgets << "as_qaction: " << type_as_string() << LF;
+
+    // DON'T: this breaks dynamic menus! cachedMenu seems to duplicate what
+    // cachedAction did before, and now cachedAction makes no sense... :(
+    //  if (cachedAction) return cachedAction;
   
   switch (type) {
     case vertical_menu:
@@ -331,7 +298,7 @@ qt_ui_element_rep::as_qaction () {
         a->setParent (m);
       }
       act->setMenu (m);
-      return act;
+      cachedAction = act;
     }
       break;
     
@@ -340,7 +307,7 @@ qt_ui_element_rep::as_qaction () {
     {
       typedef array<widget> T;
       array<widget> arr = open_box<T> (load);
-      return new QTMMinibarAction (NULL, arr);
+      cachedAction = new QTMMinibarAction (arr);
     }
       break;
       
@@ -361,7 +328,7 @@ qt_ui_element_rep::as_qaction () {
         wids[3*i+2] = ::glue_widget(false, true, 1, 1);
       }
 
-      return new QTMMinibarAction (NULL, wids);
+      cachedAction = new QTMMinibarAction (wids);
     }
       break;
 
@@ -374,8 +341,7 @@ qt_ui_element_rep::as_qaction () {
       array<widget> a = x.x1;
       int        cols = x.x2;
 
-      QWidgetAction* act= new QTMTileAction (NULL, a, cols);
-      return act;
+      cachedAction = new QTMTileAction (a, cols);
     }
       break;
       
@@ -384,29 +350,22 @@ qt_ui_element_rep::as_qaction () {
       typedef array<widget> T;
       array<widget> arr = open_box<T> (load);
 
-      QWidgetAction* act= new QTMMinibarAction (NULL, arr);
-      return act;
+      cachedAction = new QTMMinibarAction (arr);
     }
       break;
       
     case menu_separator:
         // a horizontal or vertical menu separator
     {
-      typedef bool T;
-      bool vertical = open_box<T> (load);
-
-      (void) vertical;
-      QAction* a= new QTMAction (NULL);
-      a->setSeparator (true);
-      return a;
+      cachedAction = new QTMAction (NULL);
+      cachedAction->setSeparator (true);
     }
       break;
 
     case glue_widget:
     {
-      QTMAction* a = new QTMAction();
-      a->setEnabled(false);
-      return a;
+      cachedAction = new QTMAction();
+      cachedAction->setEnabled(false);
     }
       break;
 
@@ -414,15 +373,14 @@ qt_ui_element_rep::as_qaction () {
         // a menu group; the name should be greyed and centered
     {
       typedef pair<string, int> T;
-      T x = open_box<T>(load);
+      T         x = open_box<T> (load);
       string name = x.x1;
       int   style = x.x2;  //FIXME: ignored. Use a QWidgeAction to use it?
       
-      QAction* a= new QTMAction (NULL);
-      a->setText (to_qstring (name));
-      a->setEnabled (false);
-      a->setFont (to_qfont(style, a->font())); 
-      return a;
+      cachedAction = new QTMAction (NULL);
+      cachedAction->setText (to_qstring (name));
+      cachedAction->setEnabled (false);
+      cachedAction->setFont (to_qfont (style, cachedAction->font())); 
     }
       break;
       
@@ -431,21 +389,14 @@ qt_ui_element_rep::as_qaction () {
         // a button w with a lazy pulldown menu pw
     {
       typedef pair<widget, promise<widget> > T;
-      T x = open_box<T>(load);
-      widget w = x.x1;
+      T                x = open_box<T> (load);
+      qt_widget      qtw = concrete (x.x1);
       promise<widget> pw = x.x2;
 
-      QAction*      a = concrete (w) -> as_qaction ();
+      cachedAction    = qtw->as_qaction ();
       QTMLazyMenu* lm = new QTMLazyMenu (pw);
-      QMenu* old_menu = a->menu();
-      a->setMenu (lm);
-      a->setEnabled (true);
-      if (old_menu) {
-        if (DEBUG_QT_WIDGETS)
-          cout << "qt_ui_element_rep::as_qaction(), this should not happen\n";
-        delete old_menu;
-      }
-      return a;
+      lm->attachTo (cachedAction);  // lm deletes itself after cachedAction dies
+      cachedAction->setEnabled (true);
     }
       break;
       
@@ -454,42 +405,45 @@ qt_ui_element_rep::as_qaction () {
         // keyboard shortcut; if ok does not hold, then the button is greyed
     {
       typedef quintuple<widget, command, string, string, int> T;
-      T x = open_box<T>(load);
+      T x = open_box<T> (load);
 
-      widget w    = x.x1;
-      command cmd = x.x2;
-      string pre  = x.x3;
-      string ks   = x.x4;
-      int style   = x.x5;
+      qt_widget qtw = concrete (x.x1);
+      command   cmd = x.x2;
+      string   pre  = x.x3;
+      string   ks   = x.x4;
+      int   style   = x.x5;
       
-      bool ok= (style & WIDGET_STYLE_INERT) == 0;
       QTMCommand* c;
-      QAction* a = concrete(w)->as_qaction();
+      cachedAction = qtw->as_qaction();
+        /* NOTE: we install shortcuts but in QTMWidget::event() we also handle
+         QEvent::ShortcutOverride, effectively disabling the shortcuts. This
+         allows us to keep the shortcut text in the menus while relaying all
+         keypresses through the editor.
+         */
       const QKeySequence& qks = to_qkeysequence (ks);
       if (!qks.isEmpty()) {
-        a->setShortcut (qks);
+        cachedAction->setShortcut (qks);
         command key_cmd = tm_new<qt_key_command_rep> (ks);
-        c= new QTMCommand (a, key_cmd);
+        c= new QTMCommand (cachedAction, key_cmd);
       } else {
-        c= new QTMCommand (a, cmd);
+        c= new QTMCommand (cachedAction, cmd);
       }
         
       // NOTE: this used to be a Qt::QueuedConnection, but the slot would not
       // be called if in a contextual menu
-      QObject::connect (a, SIGNAL (triggered ()), c, SLOT (apply ()));    
+      QObject::connect (cachedAction, SIGNAL (triggered()), c, SLOT (apply()));    
   
-      // FIXME: implement complete prefix handling
-      a->setEnabled (ok? true: false);
+      bool ok = (style & WIDGET_STYLE_INERT) == 0;
+      cachedAction->setEnabled (ok? true: false);
       
+        // FIXME: implement complete prefix handling
       bool check = (pre != "") || (style & WIDGET_STYLE_PRESSED);
-      
-      a->setCheckable (check? true: false);
-      a->setChecked (check? true: false);
+      cachedAction->setCheckable (check? true: false);
+      cachedAction->setChecked (check? true: false);
       if (pre == "v") {}
       else if (pre == "*") {}
       // [mi setOnStateImage:[NSImage imageNamed:@"TMStarMenuBullet"]];
       else if (pre == "o") {}
-      return a;
     }
       break;
       
@@ -499,20 +453,19 @@ qt_ui_element_rep::as_qaction () {
         // small while
     {
       typedef pair<widget, widget> T;
-      T x = open_box<T>(load);
-      widget text = x.x1;
-      widget help = x.x2;
+      T            x = open_box<T> (load);
+      qt_widget  qtw = concrete (x.x1);
+      qt_widget help = concrete (x.x2);
 
-      QAction* a= concrete(text)->as_qaction();
+      cachedAction = qtw->as_qaction();
       {
         typedef quartet<string, int, color, bool> T1;
-        T1 x = open_box<T1> (static_cast<qt_ui_element_rep*> (help.rep)->load);
-        string str = x.x1;
-        a->setToolTip (to_qstring (str));
+        T1 y = open_box<T1> (get_payload (help, text_widget));
+        cachedAction->setToolTip (to_qstring (y.x1));
         // HACK: force displaying of the tooltip (needed for items in the QMenuBar)
-        QObject::connect (a, SIGNAL(hovered()), a, SLOT(showToolTip()));
+        QObject::connect (cachedAction, SIGNAL(hovered()),
+                          cachedAction, SLOT(showToolTip()));
       }
-      return a;
     }
       break;
       
@@ -529,33 +482,31 @@ qt_ui_element_rep::as_qaction () {
       QTMAction* a = new QTMAction (NULL);
       a->set_text (str);
       a->setFont (to_qfont (style, a->font()));
-      return a;
+      cachedAction = a;
     }
       break;
       
     case xpm_widget:
         // a widget with an X pixmap icon
     {
-      url  image = open_box<url>(load);
-
-      QAction* a = new QTMAction (NULL);
-      a->setIcon (QIcon (as_pixmap (*xpm_image (image))));
-      return a;
+      url    image = open_box<url>(load);
+      cachedAction = new QTMAction (NULL);
+      cachedAction->setIcon (QIcon (as_pixmap (*xpm_image (image))));
     }
       break;
 
     default:
-      ;
+      FAILED (c_string ("qt_ui_element: unknown type for as_qaction, "
+                        * type_as_string()));
   }
-  
-  return NULL;
+  return cachedAction;
 }
 
 
 QLayoutItem *
 qt_ui_element_rep::as_qlayoutitem () {
   if (DEBUG_QT_WIDGETS)
-    cout << "as_qlayoutitem: " << type_as_string() << LF;
+    debug_widgets << "as_qlayoutitem: " << type_as_string() << LF;
 
   switch (type) {
     case horizontal_menu:
@@ -687,6 +638,7 @@ qt_ui_element_rep::as_qlayoutitem () {
     case enum_widget:
     case choice_widget:
     case filtered_choice_widget:
+    case tree_view_widget:
     case scrollable_widget:
     case hsplit_widget:
     case vsplit_widget:
@@ -697,7 +649,7 @@ qt_ui_element_rep::as_qlayoutitem () {
     case refreshable_widget:
     case balloon_widget:
     {
-      QWidgetItem* wi = new QWidgetItem(this->as_qwidget());
+      QWidgetItem* wi = new QWidgetItem (this->as_qwidget());
       return wi;
     }
       break;
@@ -708,17 +660,11 @@ qt_ui_element_rep::as_qlayoutitem () {
       typedef quartet<bool, bool, SI, SI> T;
       T x = open_box<T>(load);
 
-      /* This should be right, but actually breaks lots of stuff... ?!?
-      QSizePolicy::Policy hpolicy = x.x1 ? QSizePolicy::Minimum 
-                                         : QSizePolicy::Fixed;
-      QSizePolicy::Policy vpolicy = x.x2 ? QSizePolicy::Minimum
-                                         : QSizePolicy::Fixed;
-       */
-      QSizePolicy::Policy hpolicy = x.x1 ? QSizePolicy::Expanding 
+      QSizePolicy::Policy hpolicy = x.x1 ? QSizePolicy::MinimumExpanding
                                          : QSizePolicy::Minimum;
-      QSizePolicy::Policy vpolicy = x.x2 ? QSizePolicy::Expanding
+      QSizePolicy::Policy vpolicy = x.x2 ? QSizePolicy::MinimumExpanding
                                          : QSizePolicy::Minimum;
-      
+
       return new QSpacerItem (x.x3, x.x4, hpolicy, vpolicy);
     }
       break;
@@ -737,8 +683,12 @@ qt_ui_element_rep::as_qlayoutitem () {
 QWidget *
 qt_ui_element_rep::as_qwidget () {
   if (DEBUG_QT_WIDGETS)
-    cout << "as_qwidget: " << type_as_string() << LF;
+    debug_widgets << "as_qwidget: " << type_as_string() << LF;
 
+    // Don't return the cached widget, this only happens for refresh_widgets
+    // and then we crash.
+//  if (qwid) return qwid;
+  
   switch (type) {
     case horizontal_menu:
     case vertical_menu:
@@ -756,7 +706,8 @@ qt_ui_element_rep::as_qwidget () {
         w->setLayout(l);
       else if (DEBUG_QT_WIDGETS)
           // should we create a default layout?
-        cout << "qt_ui_element_rep::as_qwidget() : invalid situation" << LF;
+        debug_widgets << "qt_ui_element_rep::as_qwidget(): "
+                      << "invalid situation" << LF;
       qwid = w;
     }
       break;
@@ -808,19 +759,35 @@ qt_ui_element_rep::as_qwidget () {
     case pulldown_button:
     case pullright_button:
     {
-      typedef promise<widget> T1;
-      typedef pair<widget, T1> T;
-      T      x = open_box<T>(load);
-      widget w = x.x1;
-        //T1    pw = x.x2;
+      typedef pair<widget, promise<widget> > T;
+      T                x = open_box<T> (load);
+      qt_widget      qtw = concrete (x.x1);
+      promise<widget> pw = x.x2;
       
-      // a button w with a lazy pulldown menu pw
-      
-      QAction*     a = concrete(this)->as_qaction ();
-      QToolButton* b = new QTMUIButton();
-      a->setParent (b);
-      b->setDefaultAction (a);
-      qwid = b;
+      if (qtw->type == xpm_widget) {
+        url image = open_box<url> (get_payload (qtw));
+        QToolButton* b = new QToolButton();
+        
+        QTMLazyMenu* lm = new QTMLazyMenu (pw, b, type == pullright_button);
+        b->setIcon (QIcon (as_pixmap (*xpm_image (image))));
+        b->setPopupMode (QToolButton::InstantPopup);
+        b->setAutoRaise (true);
+        b->setMenu (lm);
+        qwid = b;
+      } else if (qtw->type == text_widget) {
+        typedef quartet<string, int, color, bool> T1;
+        T1 y = open_box<T1> (get_payload (qtw));
+        QPushButton* b  = new QPushButton();
+        QTMLazyMenu* lm = new QTMLazyMenu (pw, b, type == pullright_button);
+        b->setMenu (lm);
+        b->setAutoDefault (false);
+        b->setText (to_qstring (y.x1));
+        b->setEnabled (! (y.x2 & WIDGET_STYLE_INERT));
+        qt_apply_tm_style (b, y.x2, y.x3);
+        qwid = b;
+      }
+      qwid->setStyle (qtmstyle());
+
     }
       break;
       
@@ -830,31 +797,35 @@ qt_ui_element_rep::as_qwidget () {
     {
       typedef quintuple<widget, command, string, string, int> T;
       T x = open_box<T>(load);
-      widget    w = x.x1; // the button's contents: xpm_widget, text_widget, ...?
-      command cmd = x.x2;
-      string  pre = x.x3;
-      string   ks = x.x4;
-      int   style = x.x5;
+      qt_widget qtw = concrete(x.x1); // contents: xpm_widget, text_widget, ...?
+      command   cmd = x.x2;
+      string    pre = x.x3;
+      string     ks = x.x4;
+      int     style = x.x5;
       
-      QWidget* qw = concrete(w)->as_qwidget();  // will be discarded at the end
-
-      if (concrete(w)->type == xpm_widget) {  // Toolbar button
+      if (qtw->type == xpm_widget) {  // Toolbar button
         QAction*     a = as_qaction();        // Create key shortcuts and actions
-        QTMUIButton* b = new QTMUIButton();
+        QToolButton* b = new QToolButton ();
+        b->setIcon (a->icon());
+        b->setPopupMode (QToolButton::InstantPopup);
+        b->setAutoRaise (true);
         b->setDefaultAction (a);
         a->setParent (b);
         qwid = b;
       } else { // text_widget
         QPushButton*     b = new QPushButton();
-        QTMCommand* qtmcmd = new QTMCommand(b, cmd);
+        QTMCommand* qtmcmd = new QTMCommand (b, cmd);
         QObject::connect (b, SIGNAL (clicked ()), qtmcmd, SLOT (apply ()));
-        if (qw && concrete(w)->type == text_widget)
-          b->setText (static_cast<QLabel*> (qw)->text());
+        if (qtw->type == text_widget) {
+          typedef quartet<string, int, color, bool> T1;
+          b->setText (to_qstring (open_box<T1> (get_payload (qtw)).x1));
+        }
         b->setFlat (! (style & WIDGET_STYLE_BUTTON));
         qwid = b;
       }
+      qwid->setStyle (qtmstyle());
+      qt_apply_tm_style (qwid, style);
       qwid->setEnabled (! (style & WIDGET_STYLE_INERT));
-      delete qw;
     }
       break;
       
@@ -863,17 +834,14 @@ qt_ui_element_rep::as_qwidget () {
     case balloon_widget:
     {
       typedef pair<widget, widget> T;
-      T         x = open_box<T>(load);
-      widget text = x.x1;
-      widget help = x.x2;
-
-      QWidget* w = concrete(text)->as_qwidget();
-      if (w) {
-        typedef quartet<string, int, color, bool> T1;
-        T1 x = open_box<T1> (static_cast<qt_ui_element_rep*> (help.rep)->load);
-        string str = x.x1;
-        w->setToolTip (to_qstring (str));
-      }
+      T            x = open_box<T>(load);
+      qt_widget  qtw = concrete (x.x1);
+      qt_widget help = concrete (x.x2);
+      
+      typedef quartet<string, int, color, bool> T1;
+      T1 y = open_box<T1>(get_payload (help, text_widget));
+      QWidget* w = qtw->as_qwidget();
+      w->setToolTip (to_qstring (y.x1));
       qwid = w;
     }
       break;
@@ -908,7 +876,7 @@ qt_ui_element_rep::as_qwidget () {
     case xpm_widget:
     {
       url image = open_box<url>(load);
-      QLabel*   l = new QLabel (NULL);
+      QLabel* l = new QLabel (NULL);
       l->setPixmap (as_pixmap (*xpm_image (image)));
       qwid = l;
     }
@@ -924,7 +892,7 @@ qt_ui_element_rep::as_qwidget () {
       
       QCheckBox* w  = new QCheckBox (NULL);  
       w->setCheckState (check ? Qt::Checked : Qt::Unchecked);
-      qt_apply_tm_style(w, style);
+      qt_apply_tm_style (w, style);
       
       command tcmd = tm_new<qt_toggle_command_rep> (w, cmd);
       QTMCommand* c = new QTMCommand (w, tcmd);
@@ -976,7 +944,7 @@ qt_ui_element_rep::as_qwidget () {
                               x.x4);
     }
       break;
-    
+
     case filtered_choice_widget:
     {
       typedef quartet<command, array<string>, string, string> T;
@@ -986,7 +954,7 @@ qt_ui_element_rep::as_qwidget () {
                                                    QStringList (to_qstring (x.x3)),
                                                    false, true, true);
 
-      QTMLineEdit* lineEdit = new QTMLineEdit (0, "1w");
+      QTMLineEdit* lineEdit = new QTMLineEdit (0, "string", "1w");
       QObject::connect (lineEdit, SIGNAL (textChanged (const QString&)),
                         choiceWidget->filter(), SLOT (setFilterRegExp (const QString&)));
       lineEdit->setText (to_qstring (filter));
@@ -1003,6 +971,14 @@ qt_ui_element_rep::as_qwidget () {
     }
       break;
       
+    case tree_view_widget:
+    {
+      typedef triple<command, tree, tree> T;
+      T  x = open_box<T>(load);
+      qwid = new QTMTreeView (x.x1, x.x2, x.x3);  // command, data, roles
+    }
+      break;
+      
     case scrollable_widget:
     {
       typedef pair<widget, int> T;
@@ -1038,8 +1014,8 @@ qt_ui_element_rep::as_qwidget () {
       QSplitter* split = new QSplitter();
       split->setOrientation(type == hsplit_widget ? Qt::Horizontal 
                                                   : Qt::Vertical);
-      split->addWidget(qw1);
-      split->addWidget(qw2);
+      split->addWidget (qw1);
+      split->addWidget (qw2);
       
       qwid = split;
     }
