@@ -26,7 +26,8 @@
 #include <QPrinter>
 #include <QPrintDialog>
 
-#include "rgb_colors.hpp"
+#include "colors.hpp"
+
 #include "dictionary.hpp"
 #include "converter.hpp"
 #include "language.hpp"
@@ -76,6 +77,7 @@ parse_tm_style (int style) {
   if (style & WIDGET_STYLE_MINI) {  // Use smaller text font
     int fs = as_int (get_preference ("gui:mini-fontsize", QTM_MINI_FONTSIZE));
     sheet += QString("font-size: %1pt;").arg (fs > 0 ? fs : QTM_MINI_FONTSIZE);
+    sheet += QString("padding: 1px;");
   }
   if (style & WIDGET_STYLE_MONOSPACED)  // Use monospaced font
     sheet += "font-family: \"monospace\";";
@@ -106,7 +108,7 @@ qt_apply_tm_style (QWidget* qwid, int style) {
 void
 qt_apply_tm_style (QWidget* qwid, int style, color c) {
   int r,g,b,a;
-  get_rgb_color(c, r, g, b, a);
+  get_rgb_color (c, r, g, b, a);
   a = a*100/255;
   QString sheet = "* {" + parse_tm_style (style)
     + QString("color: rgba(%1, %2, %3, %4%);").arg(r).arg(g).arg(b).arg(a)
@@ -174,51 +176,46 @@ qt_decode_length (string width, string height,
 static string
 conv_sub (const string& ks) {
   string r(ks);
+  r = replace (r, "S-", "Shift+");
+  r = replace (r, "A-", "Alt+");
+  //r = replace (r, "K-", "");
 #ifdef Q_WS_MAC
-  r = replace (r, "S-", "Shift+");
   r = replace (r, "C-", "Meta+");
-  r = replace (r, "A-", "Alt+");
   r = replace (r, "M-", "Ctrl+");
-  //r = replace (r, "K-", "");
 #else
-  r = replace (r, "S-", "Shift+");
   r = replace (r, "C-", "Ctrl+");
-  r = replace (r, "A-", "Alt+");
   r = replace (r, "M-", "Meta+");
-  //r = replace (r, "K-", "");
 #endif
-  // FIXME: tokenizing is unnecessary if we ignore keysequences with spaces!
-  r = replace (r, " ", ",");
-  array<string> a = tokenize (r, ",");
+  array<string> a = tokenize (r, " ");
   for (int i = 0; i < N(a); ++i) {
-    int p = search_forwards ("+", a[i]);
-    if (p != -1 && N(a[i]) > p+1) {
-      if (is_locase (a[i][p+1]))
-        a[i] = a[i](0, p) * upcase_all (a[i] (p, N(a[i])));
-      else if (is_upcase (a[i][p+1]))
-        a[i] = a[i](0, p) * "+Shift" * upcase_all (a[i] (p, N(a[i])));
+    int pos = -1, tmp = 0, n = N(a[i]);
+    while (tmp < n && (tmp = search_forwards ("+", tmp, a[i])) != -1)
+      pos = tmp++;
+    if (pos != -1 && n > pos+1) {
+      if (a[i][pos+1] == '&')
+         // FIXME: this isn't enough to see the ampersand in the menus... (?)
+        a[i] = a[i](0, pos+1) * "&&";
+      else if (is_locase (a[i][pos+1]))
+        a[i] = a[i](0, pos) * upcase_all (a[i] (pos, n));
+      else if (is_upcase (a[i][pos+1])) {
+        // HACK: don't prepend Shift for function keys F1, F2...
+        if (n>pos+2 && a[i][pos+1] == 'F' && as_int (a[i][pos+2]) > 0)
+          ;
+        else
+          a[i] = a[i](0, pos) * "+Shift" * upcase_all (a[i] (pos, n));
+      }
+    }
   }
-}
   return recompose (a, ",");
 }
 
 QKeySequence
 to_qkeysequence (string ks) {
-    // If a key sequence defined in scheme with a kbd-map is made of several
-    // keys it contains spaces.
-    //  FIXME: Sometimes these shortcuts will conflict with normal text input
-    // (see bug #37399). Sometimes the qt_key_command won't do anything...
-    // Maybe we should bypass all this and simply add the shortcut names as text
-    // to the menu items' names...
-  if (search_forwards (" ", ks) != -1) {
-    if (DEBUG_QT) cout << "Ignoring keysequence: " << ks << LF;
-    return QKeySequence();
-  }
   string r (conv_sub (ks));
   if (DEBUG_QT && N(r) > 0) {
     QKeySequence qks (to_qstring (r));
-    cout << "ks: " << ks << " --> " << r << " --> "
-         << qks.toString (QKeySequence::NativeText).toAscii().data() << LF;
+    debug_qt << "ks: " << ks << " --> " << r << " --> "
+             << qks.toString (QKeySequence::NativeText).toAscii().data() << LF;
     return qks;
   }
   return QKeySequence (to_qstring (r));
@@ -332,36 +329,28 @@ from_qstring (const QString &s) {
   return utf8_to_cork (from_qstring_utf8 (s));
 }
 
-// Although slow to build, this should provide better lookup times than
-// linearly traversing the array of colors.
+// This should provide better lookup times
 static QHash<QString, QColor> _NamedColors;
 
-/*!
-  This needn't be called more than once. Takes RGBColors, defined in
-  rgb_colors.hpp and initializes our QHash
- */
-void initNamedColors(void) {
-  for(int i = 0; i < RGBColorsSize; ++i)
-    _NamedColors.insert(QString(RGBColors[i].name), 
-                        QColor(RGBColors[i].r, RGBColors[i].g, RGBColors[i].b));
-}
-
 /*!
   Takes either an hexadecimal RGB color, as in #e3a1ff, or a named color
-  as those defined in src/Graphics/Renderer/rgb_colors.hpp and returns a QColor
+  as those defined in src/Graphics/Renderer/ * _colors.hpp and returns a QColor
  */
 QColor
 to_qcolor (const string& col) {
   QString _col = to_qstring(col);
-  if(_col.startsWith("#"))
-    return QColor(_col);
-  if(_NamedColors.isEmpty())
-    initNamedColors();
-  if(_NamedColors.contains(_col))
+  if (_NamedColors.contains (_col))
     return _NamedColors[_col];
-  if(DEBUG_QT_WIDGETS)
-    cout << "to_qcolor(" << col << "): name is not defined in RGBColors.\n";
-  return QColor(100,100,100);  // FIXME? 
+  color c= named_color (col);
+  if (c == 0 && locase_all (col) != "black") {
+    if(DEBUG_QT_WIDGETS)
+      debug_widgets << "to_qcolor(" << col << "): "
+        << "name is not defined.\n";
+    return QColor(100,100,100);  // FIXME? 
+  }
+  QColor _c= to_qcolor (c);
+  _NamedColors.insert(_col, _c);
+  return _c;
 }
 
 /*! Returns a color encoded as a string with hexadecimal RGB values, 
@@ -372,11 +361,11 @@ from_qcolor (const QColor& col) {
   return from_qstring(col.name());
 }
 
-// FIXME: Unnecessary? QColor::setRgba() does this.
 QColor
 to_qcolor(color c) {
   int r, g, b, a;
   get_rgb_color (c, r, g, b, a);
+  if (get_reverse_colors ()) reverse (r, g, b);
   return QColor (r, g, b, a);
 }
 
@@ -384,6 +373,7 @@ color
 to_color (const QColor& c) {
   int r, g, b, a;
   c.getRgb (&r, &g, &b, &a);
+  if (get_reverse_colors ()) reverse (r, g, b);
   return rgb_color (r, g, b, a);
 }
 
@@ -414,8 +404,8 @@ qt_image_size (url image, int& w, int& h) {
       remove (temp);
     }
     else {
-      cerr << "TeXmacs] cannot read image file '" << image << "'" 
-           << " in qt_image_size" << LF;
+      convert_error << "Cannot read image file '" << image << "'" 
+                    << " in qt_image_size" << LF;
       w= 35; h= 35;
     }
   }
@@ -429,8 +419,8 @@ void
 qt_convert_image (url image, url dest, int w, int h) {
   QImage im (utf8_to_qstring (concretize (image)));
   if (im.isNull ())
-    cerr << "TeXmacs] cannot read image file '" << image << "'" 
-         << " in qt_convert_image" << LF;
+    convert_error << "Cannot read image file '" << image << "'" 
+                  << " in qt_convert_image" << LF;
   else {
     if (w > 0 && h > 0) 
       im= im.scaled (w, h, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
@@ -450,8 +440,8 @@ qt_image_to_eps (url image, int w_pt, int h_pt, int dpi) {
   QImage im (utf8_to_qstring (concretize (image)));
   string r;
   if (im.isNull ())
-    cerr << "TeXmacs Cannot read image file '" << image << "'"
-	 << " in qt_image_to_eps" << LF;
+    convert_error << "Cannot read image file '" << image << "'"
+                  << " in qt_image_to_eps" << LF;
   else {
     bool alpha= im.hasAlphaChannel ();
     if (dpi > 0 && w_pt > 0 && h_pt > 0) {
@@ -543,6 +533,58 @@ qt_image_to_eps (url image, int w_pt, int h_pt, int dpi) {
   return r;
 }
 
+
+void
+qt_image_data (url image, int& w, int&h, string& data, string& palette, string& mask) {
+  (void) palette;
+  QImage im (utf8_to_qstring (concretize (image)));
+
+  
+  
+  if (im.isNull ())
+    convert_error << "Cannot read image file '" << image << "'"
+                  << " in qt_image_data" << LF;
+  else {
+    bool alpha= im.hasAlphaChannel ();
+    w=  im.width ();
+    h=  im.height ();
+
+    data = string ((w*h)*3);
+
+    int v, i= 0, j= 0, k= 0, l= 0;
+    
+    for (j= 0; j < im.height (); j++) {
+      for (i=0; i < im.width (); i++) {
+        QRgb p= im.pixel (i, j);
+        data[l++] = qRed (p);
+        data[l++] = qGreen (p);
+        data[l++] = qBlue (p);
+      }
+    }
+    
+    if (alpha) {
+      mask = string ((w*h+7)/8);
+
+      
+      v= 0;
+      for (i=0; i < im.width (); i++) {
+        v+= (qAlpha (im.pixel (i, j)) == 0) << (3 - i % 4);
+        if (i % 8 == 7 || i + 1 == im.width ()) {
+          mask[k++] = v;
+          v= 0;
+          // Padding of the image data mask
+          if (i + 1 == im.width () && k % 2 == 1) {
+            mask[k++] = 0;
+          }
+        }
+      }
+    }
+    
+  }
+}
+
+
+
 QPixmap
 as_pixmap (const QImage& im) {
   QPixmap pm (im.size ());
