@@ -28,8 +28,6 @@
 #include "QTMMenuHelper.hpp"
 #include "QTMWindow.hpp"
 
-widget the_keyboard_focus (NULL);
-
 
 /******************************************************************************
  * qt_widget_rep: the base widget for the Qt port.
@@ -41,15 +39,17 @@ qt_widget_rep::qt_widget_rep(types _type, QWidget* _qwid)
   : widget_rep (), id (widget_counter++), qwid (_qwid), type (_type)
 {
   if (DEBUG_QT_WIDGETS)
-    cout << "qt_widget_rep: created a " << type_as_string() << LF;
+    debug_widgets << "qt_widget_rep: created a " << type_as_string() << LF;
 }
 
 qt_widget_rep::~qt_widget_rep() { 
   if (DEBUG_QT_WIDGETS)
-    cout << "~qt_widget_rep: deleted a " << type_as_string() << LF;
-  
+    debug_widgets << "~qt_widget_rep: deleted a " << type_as_string() << LF;
+
   // DON'T DO THIS! (several qt_widget_rep may have the same underlying QWidget)
-  //delete qwid; 
+  // UPD: really? when? The problem is rather order of destruction. If we
+  // completely bypassed QObject's hierarchy deleting should be ok.
+  //delete qwid;
 }
 
 void
@@ -64,15 +64,54 @@ qt_widget_rep::add_children (array<widget> a) {
 
 void
 qt_widget_rep::send (slot s, blackbox val) {
-  if (DEBUG_QT_WIDGETS)
-    cout << "qt_widget_rep::send(), unhandled " << slot_name (s)
-         << " for widget of type: " << type_as_string() << "." << LF;
+  switch (s) {
+    case SLOT_KEYBOARD_FOCUS:
+    {
+      check_type<bool> (val, s);
+      bool focus = open_box<bool> (val);
+      if (focus && qwid && !qwid->hasFocus())
+        qwid->setFocus (Qt::OtherFocusReason);
+    }
+      break;
+    case SLOT_KEYBOARD_FOCUS_ON:
+    {
+      string field = open_box<string>(val);
+      if (qwid) {
+        QWidget* target = qwid->findChild<QWidget*> (to_qstring (field));
+        if (target == NULL)
+          target = qwid->findChild<QWidget*> ("default focus target");
+        if (target) target->setFocus(Qt::OtherFocusReason);
+      }
+      /* FIXME: This would be better than using QObject::findChild but it won't
+       work because the array of children is only sloppily used at best
+       (meaning some objects are not assigned as children of others...)
+       If this is ever made to work, we'll want to handle SLOT_KEYBOARD_FOCUS_ON
+       inside qt_input_text_widget_rep and possibly others
 
-  if (s == SLOT_DESTROY) {
-    if (DEBUG_QT_WIDGETS)
-      cout << "         Resending to " << N(children) << " children" << LF;
-    for (int i = 0; i < N(children); ++i)
-      concrete (children[i])->send (s, val);
+      for (int i = 0; i < N(children); ++i)
+        if (!is_nil(children[i])) children[i]->send (s, val);
+       */
+    }
+      break;
+    case SLOT_NAME:
+    {
+        // CHECK ME!
+      widget win = qt_window_widget_rep::widget_from_qwidget (qwid);
+      if (! is_nil (win)) win->send (SLOT_NAME, val);
+    }
+      break;
+    case SLOT_DESTROY:
+    {
+      if (DEBUG_QT_WIDGETS)
+        debug_widgets << "Resending to " << N(children) << " children" << LF;
+      for (int i = 0; i < N(children); ++i)
+        if (!is_nil(children[i])) children[i]->send (s, val);
+    }
+      break;
+    default:
+      if (DEBUG_QT_WIDGETS)
+        debug_widgets << "qt_widget_rep::send(), unhandled " << slot_name (s)
+                      << " for widget of type: " << type_as_string() << ".\n";
   }
 }
 
@@ -88,7 +127,8 @@ qt_widget_rep::send (slot s, blackbox val) {
 inline QWidget*
 qt_widget_rep::as_qwidget () {
   if (DEBUG_QT_WIDGETS)
-    cout << "qt_widget_rep::as_qwidget() for " << type_as_string() << LF;
+    debug_widgets << "qt_widget_rep::as_qwidget() for "
+                  << type_as_string() << LF;
   return qwid;
 }
 
@@ -101,7 +141,7 @@ qt_widget_rep::as_qwidget () {
 QAction*
 qt_widget_rep::as_qaction() {
   QAction* a = new QTMAction (NULL); 
-  a->setEnabled(false);
+  a->setEnabled (false);
   return a;
 }
 
@@ -120,7 +160,6 @@ qt_widget_rep::as_qlayoutitem () {
  This method must not give ownership of the menu to the caller, thus
  allowing menu caching at the TeXmacs level. See the implementations in
  qt_ui_element_rep and qt_menu_rep.
- Called only by code which attaches root menus in the GUI elements.
  */
 QMenu*
 qt_widget_rep::get_qmenu () {
@@ -141,43 +180,45 @@ qt_widget_rep::get_qmenu () {
  
  The default implementation should suffice in most cases.
 
-  \param title (Often?) a title for the window.
-  \param quit Scheme closure to be executed upon close. 
-  \return The new qt_window_widget.
+ \param name A unique identifier for the window. This is *not* the window title.
+ \param quit A command to be executed when the window closes.
+ \return The new qt_window_widget.
 */
 widget
-qt_widget_rep::plain_window_widget (string title, command quit) {
+qt_widget_rep::plain_window_widget (string name, command quit) {
   if (DEBUG_QT_WIDGETS)
-    cout << "qt_widget_rep::plain_window_widget() around a " << type_as_string() << LF;
+    debug_widgets << "qt_widget_rep::plain_window_widget() around a "
+                  << type_as_string() << LF;
 
-  QTMPlainWindow* win = new QTMPlainWindow(0);
+  QTMPlainWindow* win = new QTMPlainWindow (0);
   QLayoutItem*     li = as_qlayoutitem();
   if (li) {
     QLayout* l = li->layout();
     if (! l) {
-      l = new QVBoxLayout(win);
-      l->addItem(li);
+      l = new QVBoxLayout (win);
+      l->addItem (li); // Layout owns the QLayoutItem
     }
-    win->setLayout(l);  // Transfers ownership
+    win->setLayout (l);// Transfers ownership of QWidgets in QLayoutItems to win
   } else {
     QWidget* qw = as_qwidget();
     if (qw) {
-      QLayout* l = new QVBoxLayout(win);
-      l->addWidget(qw);        // The original QWidget now belongs to the layout
-      win->setLayout(l);       // And the QLayout to the QTMPlainWindow.
+      QLayout* l = new QVBoxLayout (win);
+      win->setLayout (l); // And the QLayout to the QTMPlainWindow.
+      l->addWidget (qw);  // qw now belongs to the QWidget with the layout (win)
     } else {
-      FAILED("attempt to create a window around a nil QWidget");
+      FAILED ("attempt to create a window around a nil QWidget");
     }
   }
   
-  win->setWindowTitle (to_qstring (title));
   int l,t,r,b;
   win->layout()->getContentsMargins (&l, &t, &r, &b);
   win->layout()->setContentsMargins (l+3, t+3, r+3, b+3);
+  win->setWindowTitle (to_qstring (name));  // HACK: remove me (see bug#40837)
   
-  qt_widget wid = tm_new<qt_window_widget_rep> (win, quit);
+  qt_window_widget_rep* wid = tm_new<qt_window_widget_rep> (win, name, quit);
   wid->add_child (this);
-  return abstract (wid);
+
+  return widget (wid);
 }
 
 /*! Instantiates and returns a new widget which will act as a popup widget.
@@ -212,21 +253,23 @@ tm_ostream& operator << (tm_ostream& out, qt_widget w) {
 
 /*! Creates a decorated window using the given widget.
 
- The window will have name s, contents w and perform command q upon closing.
- 
  Each widget type may choose how to present itself as a window, by 
  reimplementing qt_widget_rep::plain_window_widget(), although the base class
  qt_widget_rep provides a default implementation which suffices in most cases.
  See its documentation.
+
+ \param w    The contents of the window.
+ \param name A unique identifier for the window. This is *not* the window title.
+ \param q    A command to be executed when the window closes.
 */
 widget
-plain_window_widget (widget w, string s, command q) {
-  widget win= concrete(w)->plain_window_widget (s, q);
-  if (s != "popup") {
+plain_window_widget (widget w, string name, command q) {
+  widget win= concrete(w)->plain_window_widget (name, q);
+  if (name != "popup") {
     int xx, yy, ww, hh;
     xx = yy = ww = hh = -1;
-    get_preferred_position (s, xx, yy);
-    get_preferred_size (s, ww, hh);
+    get_preferred_position (name, xx, yy);
+    get_preferred_size (name, ww, hh);
     if (xx != -1)
       set_position (win, xx, yy);
     if (ww != -1)
@@ -270,8 +313,8 @@ popup_widget (widget w) {
 void
 destroy_window_widget (widget w) {
   if (DEBUG_QT_WIDGETS)
-    cout << "destroy_window_widget() on "
-         << static_cast<qt_widget_rep*>(w.rep)->type_as_string() << LF;
+    debug_widgets << "destroy_window_widget() on "
+                  << static_cast<qt_widget_rep*>(w.rep)->type_as_string() << LF;
 }
 
 
@@ -416,11 +459,13 @@ widget user_canvas_widget (widget w, int style) {
   return abstract (wid);
 }
 widget resize_widget (widget w, int style, string w1, string h1,
-                      string w2, string h2, string w3, string h3) {
+                      string w2, string h2, string w3, string h3,
+                      string hpos, string vpos) {
   typedef triple<string, string, string> T1;
+  (void) hpos; (void) vpos;
   qt_widget wid = qt_ui_element_rep::create (qt_widget_rep::resize_widget,
-                                             w, style,
-                                             T1(w1, w2, w3), T1(h1, h2, h3));
+                                             w, style, T1(w1, w2, w3),
+                                                       T1(h1, h2, h3));
   wid->add_child (w);
   return abstract (wid);
 }
@@ -478,7 +523,12 @@ widget ink_widget (command cb) {
   NOT_IMPLEMENTED("Ink widget");
   (void) cb; return widget();
 }
+widget tree_view_widget (command cmd, tree data, tree actions) {
+  qt_widget wid = qt_ui_element_rep::create (qt_widget_rep::tree_view_widget,
+                                             cmd, data, actions);
+  return abstract (wid);
 
+}
   //// Widgets which are not strictly required by TeXmacs have void implementations
 
 widget empty_widget () { NOT_IMPLEMENTED("empty_widget"); return widget(); }
