@@ -251,9 +251,10 @@ user_canvas_widget (widget wid, int style) {
 }
 
 widget
-resize_widget (widget w, int style, string w1, string h1,
-               string w2, string h2, string w3, string h3) {
-  return abstract (resize_widget (concrete (w), style, w1, h1, w2, h2, w3, h3));
+resize_widget (widget w, int style, string w1, string h1, string w2, string h2,
+               string w3, string h3, string hpos, string vpos) {
+  return abstract (resize_widget (concrete (w), style,
+                                  w1, h1, w2, h2, w3, h3, hpos, vpos));
 }
 
 widget
@@ -298,6 +299,12 @@ choice_widget (command cb, array<string> vals, string v, string f) {
   return abstract (choice_wk_widget (cb, vals, v));
 }
 
+widget
+tree_view_widget (command cmd, tree data, tree data_roles) {
+    // FIXME: not implemented
+  return text_widget ("Not implemented", WIDGET_STYLE_BOLD, black);
+}
+
 widget
 file_chooser_widget (command cmd, string type, bool save) {
   return abstract (file_chooser_wk_widget (cmd, type));
@@ -365,7 +372,7 @@ destroy_window_widget (widget w) {
 void
 check_type_void (blackbox bb, string s) {
   if (!is_nil (bb)) {
-    cerr << "\nslot type= " << s << "\n";
+    failed_error << "slot type= " << s << "\n";
     FAILED ("type mismatch");
   }
 }
@@ -373,7 +380,7 @@ check_type_void (blackbox bb, string s) {
 template<class T> void
 check_type (blackbox bb, string s) {
   if (type_box (bb) != type_helper<T>::id) {
-    cerr << "\nslot type= " << s << "\n";
+    failed_error << "slot type= " << s << "\n";
     FAILED ("type mismatch");
   }
 }
@@ -401,7 +408,7 @@ principal_widget_check (wk_widget wid) {
   // Currently, we only allow geometry access of the unique child of
   // a window widget.
   if (bad_parent (wid)) {
-    cerr << "Widget= " << wid << "\n";
+    failed_error << "Widget= " << wid << "\n";
     FAILED ("invalid geometry access");
   }
 }
@@ -542,6 +549,14 @@ send_keyboard_focus (wk_widget w, blackbox val) {
   w->win->set_keyboard_focus (abstract (w), open_box<bool> (val));
 }
 
+void
+send_keyboard_focus_on (wk_widget w, blackbox val) {
+  ASSERT (type_box (val) == type_helper<string>::id, "type mismatch");
+  bool done= false;
+  //w->win->set_keyboard_focus (abstract (w), true);
+  w << emit_keyboard_focus_on (open_box<string> (val), done);
+}
+
 void
 send_mouse (wk_widget w, blackbox val) {
   typedef quintuple<string,SI,SI,int,time_t> mouse;
@@ -622,6 +637,10 @@ wk_widget_rep::send (slot s, blackbox val) {
   case SLOT_NAME:
     send_string (THIS, "window name", val);
     break;
+  case SLOT_MODIFIED:
+    check_type<bool> (val, "SLOT_MODIFIED");
+    win->set_modified (open_box<bool> (val));
+    break;
   case SLOT_SIZE:
     send_size (THIS, val);
     break;
@@ -640,6 +659,9 @@ wk_widget_rep::send (slot s, blackbox val) {
   case SLOT_KEYBOARD_FOCUS:
     send_keyboard_focus (THIS, val);
     break;
+  case SLOT_KEYBOARD_FOCUS_ON:
+    send_keyboard_focus_on (THIS, val);
+    break;
   case SLOT_MOUSE:
     send_mouse (THIS, val);
     break;
@@ -700,6 +722,9 @@ wk_widget_rep::send (slot s, blackbox val) {
   case SLOT_SIDE_TOOLS_VISIBILITY:
     send_bool (THIS, "side tools", val);
     break;
+  case SLOT_BOTTOM_TOOLS_VISIBILITY:
+    send_bool (THIS, "bottom tools", val);
+    break;
   case SLOT_FOOTER_VISIBILITY:
     send_bool (THIS, "footer flag", val);
     break;
@@ -852,6 +877,8 @@ wk_widget_rep::query (slot s, int type_id) {
     return query_bool (THIS, "user icons", type_id);
   case SLOT_SIDE_TOOLS_VISIBILITY:
     return query_bool (THIS, "side tools", type_id);
+  case SLOT_BOTTOM_TOOLS_VISIBILITY:
+    return query_bool (THIS, "bottom tools", type_id);
   case SLOT_FOOTER_VISIBILITY:
     return query_bool (THIS, "footer flag", type_id);
   case SLOT_INTERACTIVE_MODE:
@@ -963,6 +990,10 @@ wk_widget_rep::write (slot s, blackbox index, widget w) {
     check_type_void (index, "SLOT_SIDE_TOOLS");
     THIS << set_widget ("side tools", concrete (w));
     break;
+  case SLOT_BOTTOM_TOOLS:
+    check_type_void (index, "SLOT_BOTTOM_TOOLS");
+    THIS << set_widget ("bottom tools", concrete (w));
+    break;
   case SLOT_SCROLLABLE:
     check_type_void (index, "SLOT_SCROLLABLE");
     THIS << set_widget ("scrollable", concrete (w));
