@@ -27,8 +27,8 @@ void mac_fix_paths ();
 #endif
 
 #ifdef QTTEXMACS
+#include "Qt/QTMApplication.hpp"
 #include "Qt/qt_utilities.hpp"
-#include <QApplication>
 #include <QDir>
 #endif
 
@@ -41,7 +41,6 @@ void mac_fix_paths ();
 #endif
 
 extern bool   char_clip;
-extern bool   reverse_colors;
 
 extern url    tm_init_file;
 extern url    tm_init_buffer_file;
@@ -150,8 +149,8 @@ TeXmacs_init_paths (int& argc, char** argv) {
   
   if (get_env ("TEXMACS_PATH") == "")
     set_env ("TEXMACS_PATH", as_string (exedir * ".."));
-  if (get_env ("HOME") == "")
-    set_env ("HOME", get_env("USERPROFILE"));
+//  if (get_env ("HOME") == "") //now set in immediate_options otherwise --setup option fails
+//    set_env ("HOME", get_env("USERPROFILE"));
   // HACK
   // In WINE the variable PWD is already in the outer Unix environment 
   // so we need to override it to have a correct behaviour
@@ -259,7 +258,7 @@ TeXmacs_main (int argc, char** argv) {
       else if ((s == "-q") || (s == "-quit"))
 	my_init_cmds= my_init_cmds * " (quit-TeXmacs)";
       else if ((s == "-r") || (s == "-reverse"))
-	reverse_colors= true;
+	set_reverse_colors(true);
       else if ((s == "-c") || (s == "-convert")) {
 	i+=2;
 	if (i<argc) {
@@ -309,11 +308,11 @@ TeXmacs_main (int argc, char** argv) {
     }
   if (flag) debug (DEBUG_FLAG_AUTO, true);
 
-  if (DEBUG_STD) cout << "TeXmacs] Installing internal plug-ins...\n";
+  if (DEBUG_STD) debug_boot << "Installing internal plug-ins...\n";
   bench_start ("initialize plugins");
   init_plugins ();
   bench_cumul ("initialize plugins");
-  if (DEBUG_STD) cout << "TeXmacs] Opening display...\n";
+  if (DEBUG_STD) debug_boot << "Opening display...\n";
   
 #if defined(X11TEXMACS) && defined(MACOSX_EXTENSIONS)
   init_mac_application ();
@@ -321,7 +320,7 @@ TeXmacs_main (int argc, char** argv) {
     
   gui_open (argc, argv);
   set_default_font (the_default_font);
-  if (DEBUG_STD) cout << "TeXmacs] Starting server...\n";
+  if (DEBUG_STD) debug_boot << "Starting server...\n";
   { // opening scope for server sv
   server sv;
 
@@ -337,7 +336,7 @@ TeXmacs_main (int argc, char** argv) {
     string s= argv[i];
     if ((N(s)>=2) && (s(0,2)=="--")) s= s (1, N(s));
     if ((s[0] != '-') && (s[0] != '+')) {
-      if (DEBUG_STD) cout << "TeXmacs] Loading " << s << "...\n";
+      if (DEBUG_STD) debug_boot << "Loading " << s << "...\n";
       url u= url_system (s);
       if (!is_rooted (u)) u= resolve (url_pwd (), "") * u;
       string b= scm_quote (as_string (u));
@@ -354,7 +353,7 @@ TeXmacs_main (int argc, char** argv) {
              (s == "-log-file")) i++;
   }
   if (install_status == 1) {
-    if (DEBUG_STD) cout << "TeXmacs] Loading welcome message...\n";
+    if (DEBUG_STD) debug_boot << "Loading welcome message...\n";
     url u= "tmfs://help/plain/tm/doc/about/welcome/first.en.tm";
     string b= scm_quote (as_string (u));
     string cmd= "(load-buffer " * b * " " * where * ")";
@@ -362,7 +361,7 @@ TeXmacs_main (int argc, char** argv) {
     exec_delayed (scheme_cmd (cmd));
   }
   else if (install_status == 2) {
-    if (DEBUG_STD) cout << "TeXmacs] Loading upgrade message...\n";
+    if (DEBUG_STD) debug_boot << "Loading upgrade message...\n";
     url u= "tmfs://help/plain/tm/doc/about/changes/changes-recent.en.tm";
     string b= scm_quote (as_string (u));
     string cmd= "(load-buffer " * b * " " * where * ")";
@@ -370,7 +369,7 @@ TeXmacs_main (int argc, char** argv) {
     exec_delayed (scheme_cmd (cmd));
   }
   if (number_buffers () == 0) {
-    if (DEBUG_STD) cout << "TeXmacs] Creating 'no name' buffer...\n";
+    if (DEBUG_STD) debug_boot << "Creating 'no name' buffer...\n";
     open_window ();
   }
 
@@ -379,22 +378,22 @@ TeXmacs_main (int argc, char** argv) {
   bench_reset ("initialize plugins");
   bench_reset ("initialize scheme");
 
-  if (DEBUG_STD) cout << "TeXmacs] Starting event loop...\n";
+  if (DEBUG_STD) debug_boot << "Starting event loop...\n";
   texmacs_started= true;
   if (!disable_error_recovery) signal (SIGSEGV, clean_exit_on_segfault);
   gui_start_loop ();
 
-  if (DEBUG_STD) cout << "TeXmacs] Stopping server...\n";
+  if (DEBUG_STD) debug_boot << "Stopping server...\n";
   } // ending scope for server sv
 
-  if (DEBUG_STD) cout << "TeXmacs] Closing display...\n";
+  if (DEBUG_STD) debug_boot << "Closing display...\n";
   gui_close ();
   
 #if defined(X11TEXMACS) && defined(MACOSX_EXTENSIONS)
   finalize_mac_application ();
 #endif
   
-  if (DEBUG_STD) cout << "TeXmacs] Good bye...\n";
+  if (DEBUG_STD) debug_boot << "Good bye...\n";
 }
 
 /******************************************************************************
@@ -431,9 +430,11 @@ void
 immediate_options (int argc, char** argv) {
   if (get_env ("TEXMACS_HOME_PATH") == "")
 #ifdef __MINGW32__
-    set_env ("TEXMACS_HOME_PATH", get_env ("APPDATA") * "/TeXmacs");
+    if (get_env ("HOME") == "")
+        set_env ("HOME", get_env("USERPROFILE"));
+    set_env ("TEXMACS_HOME_PATH", get_env ("APPDATA") * "\\TeXmacs");
 #else
-  set_env ("TEXMACS_HOME_PATH", get_env ("HOME") * "/.TeXmacs");
+    set_env ("TEXMACS_HOME_PATH", get_env ("HOME") * "/.TeXmacs");
 #endif
   if (get_env ("TEXMACS_HOME_PATH") == "") return;
   for (int i=1; i<argc; i++) {
@@ -476,9 +477,11 @@ immediate_options (int argc, char** argv) {
     else if (s == "-log-file" && i + 1 < argc) {
       i++;
       char* log_file = argv[i];
-      FILE* f= fopen (log_file, "w");
-      if (! f || ! cout.open (f) || ! cerr.open (f))
+      tm_ostream logf (log_file);
+      if (!logf->is_writable ())
         cerr << "TeXmacs] Error: could not open " << log_file << "\n";
+      cout.redirect (logf);
+      cerr.redirect (logf);
     }
   }
 }
@@ -507,7 +510,7 @@ main (int argc, char** argv) {
     // Work around Qt bug: https://bugreports.qt-project.org/browse/QTBUG-32789
   QFont::insertSubstitution (".Lucida Grande UI", "Lucida Grande");
 #endif
-  new QApplication (argc, argv);
+  new QTMApplication (argc, argv);
 #endif
   TeXmacs_init_paths (argc, argv);
   //cout << "Bench  ] Started TeXmacs\n";
@@ -524,7 +527,5 @@ main (int argc, char** argv) {
 //  test_environments ();
 //#endif
   start_scheme (argc, argv, TeXmacs_main);
-  cout.close ();
-  cerr.close ();
   return 0;
 }
