@@ -413,63 +413,63 @@ box_rep::get_info (tree in) {
 
 int
 box_rep::get_leaf_left_pos () {
-  cerr << "\nTeXmacs] the box is " << box (this) << "\n";
+  failed_error << "The box is " << box (this) << "\n";
   FAILED ("this box is not textual");
   return 0;
 }
 
 int
 box_rep::get_leaf_right_pos () {
-  cerr << "\nTeXmacs] the box is " << box (this) << "\n";
+  failed_error << "The box is " << box (this) << "\n";
   FAILED ("this box is not textual");
   return 0;
 }
 
 string
 box_rep::get_leaf_string () {
-  cerr << "\nTeXmacs] the box is " << box (this) << "\n";
+  failed_error << "The box is " << box (this) << "\n";
   FAILED ("this box is not textual");
   return "";
 }
 
 font
 box_rep::get_leaf_font () {
-  cerr << "\nTeXmacs] the box is " << box (this) << "\n";
+  failed_error << "The box is " << box (this) << "\n";
   FAILED ("this box is not textual");
   return font ();
 }
 
 pencil
 box_rep::get_leaf_pencil () {
-  cerr << "\nTeXmacs] the box is " << box (this) << "\n";
+  failed_error << "The box is " << box (this) << "\n";
   FAILED ("this box is not textual");
   return pencil (false);
 }
 
 language
 box_rep::get_leaf_language () {
-  cerr << "\nTeXmacs] the box is " << box (this) << "\n";
+  failed_error << "The box is " << box (this) << "\n";
   FAILED ("this box is not textual");
   return language ();
 }
 
 tree
 box_rep::get_leaf_tree () {
-  cerr << "\nTeXmacs] the box is " << box (this) << "\n";
+  failed_error << "The box is " << box (this) << "\n";
   FAILED ("no tree attached to this box");
   return "";
 }
 
 box
 box_rep::get_leaf_box () {
-  cerr << "\nTeXmacs] the box is " << box (this) << "\n";
+  failed_error << "The box is " << box (this) << "\n";
   FAILED ("no box attached to this box");
   return box ();
 }
 
 lazy
 box_rep::get_leaf_lazy () {
-  cerr << "\nTeXmacs] the box is " << box (this) << "\n";
+  failed_error << "The box is " << box (this) << "\n";
   FAILED ("no lazy attached to this box");
   return lazy ();
 }
@@ -512,7 +512,7 @@ box_rep::reindex (int i, int item, int n) {
 
 void
 box_rep::redraw (renderer ren, path p, rectangles& l) {
-  if (((nr_painted&15) == 15) && gui_interrupted (true)) return;
+  if ((nr_painted&15) == 15 && ren->is_screen && gui_interrupted (true)) return;
   ren->move_origin (x0, y0);
   SI delta= ren->pixel; // adjust visibility to compensate truncation
   if (ren->is_visible (x3- delta, y3- delta, x4+ delta, y4+ delta)) {
@@ -538,7 +538,7 @@ box_rep::redraw (renderer ren, path p, rectangles& l) {
       }
     }
     
-    if (((nr_painted&15) == 15) && gui_interrupted ()) {
+    if ((nr_painted&15) == 15 && ren->is_screen && gui_interrupted ()) {
       l= translate (l, -ren->ox, -ren->oy);
       clear_incomplete (l, ren->pixel, item, i1, i2);
       l= translate (l, ren->ox, ren->oy);
