@@ -564,47 +564,6 @@ public:
   }
 };
 
-/******************************************************************************
-* action boxes
-******************************************************************************/
-
-struct action_box_rep: public change_box_rep {
-  tree    filter; // which events are accepted ?
-  command cmd;    // command to be executed
-  path    vip;    // store this location before execution
-  action_box_rep (path ip, box b, tree f, command c, bool ch, path vip);
-  operator tree () { return tree (TUPLE, "action", bs[0]); }
-  box adjust_kerning (int mode, double factor);
-  tree action (tree t, SI x, SI y, SI delta);
-};
-
-action_box_rep::action_box_rep (
-  path ip, box b, tree filter2, command cmd2, bool child_flag, path vip2):
-  change_box_rep (ip, child_flag), filter (filter2), cmd (cmd2), vip (vip2)
-{
-  insert (b, 0, 0);
-  position ();
-  left_justify ();
-  finalize ();
-}
-
-box
-action_box_rep::adjust_kerning (int mode, double factor) {
-  box body= bs[0]->adjust_kerning (mode, factor);
-  return action_box (ip, body, filter, cmd, child_flag, vip);
-}
-
-tree
-action_box_rep::action (tree t, SI x, SI y, SI delta) {
-  if (t == filter) {
-    call ("action-set-path", reverse (vip));
-    // FIXME: we should also reset the action path
-    cmd ();
-    return "done";
-  }
-  return change_box_rep::action (t, x, y, delta);
-}
-
 /******************************************************************************
 * locus boxes
 ******************************************************************************/
@@ -660,7 +619,7 @@ locus_box_rep::loci (SI x, SI y, SI delta, list<string>& l, rectangles& rs) {
 void
 locus_box_rep::post_display (renderer &ren) {
   if (ref!="") ren->href(ref, x1, y1, x2, y2);
-  if (anchor!="") ren->anchor(anchor, x1, y1);
+  if (anchor!="") ren->anchor(anchor, x1, y1, x2, y2);
 }
 
 /******************************************************************************
@@ -669,7 +628,8 @@ locus_box_rep::post_display (renderer &ren) {
 
 struct tag_box_rep: public change_box_rep {
   tree keys;
-  tag_box_rep (path ip, box b, tree keys);
+  path tip;
+  tag_box_rep (path ip, path tip, box b, tree keys);
   operator tree () { return tree (TUPLE, "tag", bs[0]); }
   box adjust_kerning (int mode, double factor);
   tree tag (tree t, SI x, SI y, SI delta);
@@ -677,7 +637,7 @@ struct tag_box_rep: public change_box_rep {
   path find_tag (string name);
 };
 
-tag_box_rep::tag_box_rep (path ip, box b, tree keys2):
+tag_box_rep::tag_box_rep (path ip, path tip, box b, tree keys2):
   change_box_rep (ip, false), keys (keys2)
 {
   insert (b, 0, 0);
@@ -689,7 +649,7 @@ tag_box_rep::tag_box_rep (path ip, box b, tree keys2):
 box
 tag_box_rep::adjust_kerning (int mode, double factor) {
   box body= bs[0]->adjust_kerning (mode, factor);
-  return tag_box (ip, body, keys);
+  return tag_box (ip, tip, body, keys);
 }
 
 void
@@ -703,7 +663,7 @@ path
 tag_box_rep::find_tag (string search) {
   for (int i=0; i<N(keys); i++)
     if (keys[i]->label == search)
-      return reverse (descend_decode (ip, 1));
+      return reverse (descend_decode (tip, 1));
   return path ();
 }
 
@@ -891,16 +851,6 @@ remember_box (path ip, box b) {
   return tm_new<remember_box_rep> (ip, b);
 }
 
-box
-action_box (path ip, box b, tree filter, command cmd, bool ch, path vip) {
-  return tm_new<action_box_rep> (ip, b, filter, cmd, ch, vip);
-}
-
-box
-action_box (path ip, box b, tree filter, command cmd, bool ch) {
-  return tm_new<action_box_rep> (ip, b, filter, cmd, ch, decorate ());
-}
-
 box
 locus_box (path ip, box b, list<string> ids, SI pixel) {
   return tm_new<locus_box_rep> (ip, b, ids, pixel);
@@ -912,8 +862,8 @@ locus_box (path ip, box b, list<string> ids, SI pixel, string ref, string anchor
 }
 
 box
-tag_box (path ip, box b, tree keys) {
-  return tm_new<tag_box_rep> (ip, b, keys);
+tag_box (path ip, path tip, box b, tree keys) {
+  return tm_new<tag_box_rep> (ip, tip, b, keys);
 }
 
 box
