@@ -84,14 +84,15 @@ concater_rep::flag_ok (string s, path ip, color col) {
   SI h= 4*env->fn->wfn/5;
   int r, g, b, a;
   get_rgb_color (col, r, g, b, a);
-  r= 255- (255 - r)/6;
-  g= 255- (255 - g)/6;
-  b= 255- (255 - b)/6;
+  //r= 255- (255 - r)/6;
+  //g= 255- (255 - g)/6;
+  //b= 255- (255 - b)/6;
+  a= a/6;
   color light= rgb_color (r, g, b, a);
   int info= env->info_level;
   if (info == INFO_MINIMAL || info == INFO_SHORT) {
     box infob= info_box (dip, h, pencil (col, env->fn->wline), light);
-    box specb= specific_box (ip, infob, false, env->fn);
+    box specb= specific_box (ip, infob, "screen", env->fn);
     print (specb);
   }
   else if (info == INFO_DETAILED || info == INFO_PAPER) {
@@ -100,7 +101,7 @@ concater_rep::flag_ok (string s, path ip, color col) {
     box textb= text_box (decorate (ip), 0, s, gfn, col);
     box flagb= flag_box (dip, textb, h, pencil (col, env->fn->wline), light);
     if (info == INFO_DETAILED) {
-      box specb= specific_box (ip, flagb, false, env->fn);
+      box specb= specific_box (ip, flagb, "screen", env->fn);
       print (specb);
     }
     else {
@@ -611,6 +612,7 @@ concater_rep::typeset (tree t, path ip) {
   case LINK:
   case URL:
   case SCRIPT:
+  case FIND_ACCESSIBLE:
     typeset_inactive (t, ip);
     break;
   case HLINK:
