@@ -51,6 +51,9 @@ initialize_default_env () {
   env (THE_LABEL)        = "?";         // value of the next label
   env (THE_TAGS)         = tree(TUPLE); // current tags
   env (THE_MODULES)      = tree(TUPLE); // necessary modules and plug-ins
+  env (GLOBAL_TITLE)     = "";          // global document title
+  env (GLOBAL_AUTHOR)    = "";          // global document author
+  env (GLOBAL_SUBJECT)   = "";          // global document subject
 
   env (FONT)             = "roman";     // the font name in text mode
   env (FONT_FAMILY)      = "rm";        // the font family in text mode
@@ -96,7 +99,7 @@ initialize_default_env () {
   env (PAR_FLEXIBILITY)  = "1000";      // threshold for switching to ragged
   env (PAR_HYPHEN)       = "professional"; // quality of hyphenation
   env (PAR_SPACING)      = "plain";     // spacing mode (for CJK)
-  env (PAR_KERNING_STRETCH)= "0";       // extra kerning around characters
+  env (PAR_KERNING_STRETCH)= "auto";    // extra kerning around characters
   env (PAR_KERNING_MARGIN) = "false";   // use marginal kerning (protrusion)
   env (PAR_WIDTH)        = "auto";      // width of paragraph
   env (PAR_LEFT)         = "0cm";       // left indentation
@@ -269,6 +272,7 @@ initialize_default_env () {
   env (ORNAMENT_SHAPE)        = "classic";      // shape of the ornament
   env (ORNAMENT_TITLE_STYLE)  = "classic";      // title style
   env (ORNAMENT_BORDER)       = "1ln";          // border width
+  env (ORNAMENT_SWELL)        = "0.5";          // border swell
   env (ORNAMENT_HPADDING)     = "1spc";         // horizontal padding of body
   env (ORNAMENT_VPADDING)     = "1spc";         // vertical padding of body
   env (ORNAMENT_COLOR)        = "";             // background color
@@ -298,14 +302,18 @@ initialize_default_env () {
   tree tagflag (FLAG, tree (ARG, "Id"), "blue", "Id");
   tree reftxt (GET_BINDING, tree (ARG, "Id"));
   tree preftxt (GET_BINDING, tree (ARG, "Id"), "1");
+  tree act_id (ID, tree (HARD_ID, tree (ARG, "args", "0")));
+  tree act_script (MAP_ARGS, "find-accessible", "script", "args", "1");
+  tree act_ln (LINK, "action", copy (act_id), copy (act_script));
   env ("hlink")= tree (MACRO, "body", "destination",
 		       tree (LOCUS, copy (src_id), ln1,
                              tree (ARG, "body")));
-  env ("action")= tree (MACRO, "body", "destination", "where",
-			tree (LOCUS, copy (src_id), ln2,
-                              tree (ARG, "body")));
+  env ("action")= tree (XMACRO, "args",
+			tree (LOCUS, copy (act_id), copy (act_ln),
+                              tree (ARG, "args", "0")));
   env ("label")= tree (MACRO, "Id", 
-		       tree (LOCUS, copy (ref_id), ln4, tree (CONCAT, labflag, labtxt)));
+		       tree (LOCUS, copy (ref_id), ln4,
+			     tree (CONCAT, labflag, labtxt)));
   env ("tag")= tree (MACRO, "Id", "body",
 		     tree (WITH, "the-tags", merged,
 			   tree (SURROUND, tagflag, "",
